/* Name: align.m                                                             **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse Alignment, die eine Alignierung von   **
**           Sequenzen repraesentiert                                        **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 25.07.1995                                                      **
** Zuletzt bearbeitet: 20.05.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "align.h"

#include "error.h"
#include "constant.h"

#include <string.h>
#include <stdlib.h>

/* Klasse: Alignment                                                         */

@implementation Alignment

+ new:(char**)align0 number:(int)num0 length:(long)len0
{
  return [[self alloc] init:align0 number:num0 length:len0];
}

+ alloc
{
  Alignment *newObj;
  int i;
  newObj=[super alloc];
  newObj->alignment=(char**)malloc(MAXALIGNNLETS*sizeof(char*));
  for(i=0;i<MAXALIGNNLETS;i++)
    newObj->alignment[i]=(char*)malloc(MINALIGNLEN*sizeof(char));
  newObj->qualityRow=(int*)malloc(MINALIGNLEN*sizeof(int));
  if(!newObj->alignment[MAXALIGNNLETS-1] || !newObj->qualityRow)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

+ (BOOL)isGap:(char)char0
{
  if(char0==ALIGNGAPCHAR)
    return YES;
  else
    return NO;
}

- init:(char**)align0 number:(int)num0 length:(long)len0
{
  int i, j;
  self=[super init];
  number=num0;
  length=len0;
  if(number<0 || length<0)
    {
      [ErrorHandler error:ERROBJINIT];
      return self;
    }
  alignment=(char**)realloc(alignment,number*sizeof(char*));
  for(i=0;i<number;i++)
    alignment[i]=(char*)realloc(alignment[i],(length+1)*sizeof(char));
  qualityRow=(int*)realloc(qualityRow,length*sizeof(int));
  if(!alignment[number-1] || !qualityRow)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return self;
    }
  for(i=0;i<number;i++)
    if(strlen(align0[i])>=length)
      {
	strncpy(alignment[i],align0[i],length);
	alignment[i][length]='\0';
      }
    else
      {
	[ErrorHandler error:ERROBJINIT];
	return self;
      }
  quality=0;
  for(i=0;i<number;i++)
    {
      qualityRow[i]=DEFALIGNQUALITY;
      quality+=qualityRow[i];
    }
  return self;
}

- setRow:(int)row0 as:(char*)vector0
{
  if(strlen(vector0)>=length && row0>=0 && row0<number)
    {
      strncpy(alignment[row0],vector0,length);
      alignment[row0][length]='\0';
    }
  return self;
}

- setColumn:(long)col0 as:(char*)vector0
{
  int i;
  if(strlen(vector0)>=number && col0>=0 && col0<length)
    for(i=0;i<number;i++)
      alignment[i][col0]=vector0[i];
  return self;
}

- setPositionAtRow:(int)row0 column:(long)col0 as:(char)char0
{
  if(row0>=0 && row0<number && col0>=0 && col0<length)
    alignment[row0][col0]=char0;
  return self;
}

- setQualityRow:(int*)vector0
{
  int i;
  quality=0;
  for(i=0;i<length;i++)
    {
      if(vector0+i)
	qualityRow[i]=vector0[i];
      else
	{
	  qualityRow[i]=DEFALIGNQUALITY;
	  quality+=qualityRow[i];
	  [ErrorHandler error:ERROBJSETINSTVAR];
	  return self;
	}
      quality+=qualityRow[i];
    }
  return self;
}

- setQualityRowAtColumn:(long)col0 as:(int)quality0
{
  if(col0>=0 && col0<length)
    {
      quality-=qualityRow[col0];
      qualityRow[col0]=quality0;
      quality+=qualityRow[col0];
    }
  return self;
}

- (char**)getAlignment
{
  return [self getAlignmentFrom:0 to:length-1];
}

- (char**)getAlignmentFrom:(long)i0 to:(long)j0
{
  char **returnAlignment;
  long from, to;
  int i;
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=length)
    to=length-1;
  else
    to=j0;
  if(from<=to)
    {
      returnAlignment=(char**)malloc(number*sizeof(char*));
      for(i=0;i<number;i++)
	returnAlignment[i]=(char*)malloc((to-from+2)*sizeof(char));
      if(!returnAlignment[number-1])
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return 0;
	}
      for(i=0;i<number;i++)
	{
	  strncpy(returnAlignment[i],alignment[i]+from,to-from+1);
	  returnAlignment[i][length]='\0';
	}
      return returnAlignment;
    }
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}

- (char*)getRow:(int)row0
{
  char *returnRow;
  if(row0>=0 && row0<number)
    {
      returnRow=(char*)malloc((length+1)*sizeof(char));
      if(!returnRow)
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return 0;
	}
      strcpy(returnRow,alignment[row0]);
      returnRow[length]='\0';
      return returnRow;
    }
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}

- (char*)getRow:(int)row0 from:(long)i0 to:(long)j0
{
  char *returnRow;
  long from, to;
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=length)
    to=length-1;
  else
    to=j0;
  if(row0>=0 && row0<number && from<=to)
    {
      returnRow=(char*)malloc((to-from+2)*sizeof(char));
      if(!returnRow)
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return 0;
	}
      strncpy(returnRow,alignment[row0]+from,to-from+1);
      returnRow[to-from+1]='\0';
      return returnRow;
    }
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}

- (char*)getColumn:(long)col0
{
  char *returnColumn;
  int i;
  if(col0>=0 && col0<length)
    {
      returnColumn=(char*)malloc((number+1)*sizeof(char));
      if(!returnColumn)
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return 0;
	}
      for(i=0;i<number;i++)
	returnColumn[i]=alignment[i][col0];
      returnColumn[number]='\0';
      return returnColumn;
    }
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}
- (char*)getColumn:(long)col0 from:(long)i0 to:(long)j0
{
  char *returnColumn;
  long from, to;
  int i;
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=number)
    to=number-1;
  else
    to=j0;
  if(col0>=0 && col0<length && from<=to)
    {
      returnColumn=(char*)malloc((to-from+2)*sizeof(char));
      if(!returnColumn)
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return 0;
	}
      for(i=from;i<to-from+1;i++)
	returnColumn[i]=alignment[i][col0];
      returnColumn[to-from+1]='\0';
      return returnColumn;
    }
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}

- (char)getAlignmentAtRow:(int)row0 column:(long)col0
{
  if(row0>=0 && row0<number && col0>=0 && col0<length)
    return alignment[row0][col0];
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return ALIGNGAPCHAR;
    }
  return ALIGNGAPCHAR;
}

- (int)getNumber
{
  return number;
}

- (long)getLength
{
  return length;
}

- (int*)getQualityRow
{
  int *returnRow;
  int j;
  returnRow=(int*)malloc(length*sizeof(int));
  if(!returnRow)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  for(j=0;j<length;j++)
    returnRow[j]=qualityRow[j];
  return returnRow;
}

- (int)getQualityRowAtColumn:(long)col0
{
  if(col0>=0 && col0<length)
    return qualityRow[col0];
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return DEFALIGNQUALITY;
    }
  return DEFALIGNQUALITY;
}

- (long)getQuality
{
  return quality;
}

- free
{
  int i;
  for(i=0;i<number;i++)
    free(alignment[i]);
  free(alignment);
  free(qualityRow);
  return [super free];
}

@end
