/* Name: db_iface.h                                                          **
**                                                                           **
**                                                                           **
** Funktion: Schnittstellendeklaration fuer C-Anwendungen zu                 **
**           Datenbanksystemen                                               **
**                                                                           **
** Aufruf: -- (Einbindung in C-Texte ueber #include)                         **
**                                                                           **
** Probleme: Verwendung von K&R C wurde aus Kompiliergruenden notwendig      **
**                                                                           **
** Erstellt: 03.07.1995                                                      **
** Zuletzt bearbeitet: 20.10.1995                                            */

/* ------------------------------------------------------------------------- */


/* Schnittstellenfunktionen                                                  */

/* db_connect: Herstellen einer Verbindung zum DB-Server                     **
**                                                                           **
** int db_connect(char* connection_name,                                     **
**                char* user, char* password, char* server);                 */

int db_connect();


/* db_disconnect: Loesen einer Verbindung zum DB-Server                      **
** int db_disconnect(char* connection_name);                                 */

int db_disconnect();


/* db_select: Abfrage an den DB-Server                                       **
**                                                                           **
** Eine eigentliche Prozedur db_select() existiert nicht, die                **
** SELECT-Anweisung wird ueber Cursor realisiert. Mit einem Cursor kann      **
** eine SELECT-Anweisung zeilenweise ausgefuehrt werden. Ein Cursor muss     **
** deklariert, geoeffnet und geschlossen werden. Mit der Anweisung FETCH     **
** wird der aktuelle Datensatz zurueckgeliefert und der Cursor zum           **
** naechsten Satz bewegt.                                                    **
**                                                                           **
** int db_declare_cursor(char* connection_name,                              **
**                       int cursor_number, char* select_statement);         **
** int db_open_cursor(char* connection_name,                                 **
**                    int cursor_number);                                    **
** int db_fetch(char* connection_name,                                       **
**              int cursor_number,                                           **
**              int column_number, char** host_variables);                   **
** int db_close_cursor(char* connection_name,                                **
**                     int  cursor_number);                                  */

int db_declare_cursor();
int db_open_cursor();
int db_fetch();
int db_close_cursor();

/* db_insert: Einfuegen von Saetzen in eine Tabelle                          **
**            Standardmaessig durch Angabe von Werten                        **
**            Alias fuer db_insert_vaules                                    **
**                                                                           **
** int db_insert(char* connection_name,                                      **
**               char* table, char** columns, char** values);                */

int db_insert();


/* db_insert_vaules: Einfuegen von Saetzen in eine Tabelle                   **
**                   durch Angabe von Werten                                 **
**                                                                           **
** int db_insert_values(char* connection_name,                               **
**                      char* table, char** columns, char** values);         */

int db_insert_values();


/* db_insert_select: Einfuegen von Saetzen in eine Tabelle                   **
**                   durch SELECT-Anweisung                                  **
**                                                                           **
** int db_insert_select(char* connection_name,                               **
**                      char* table, char** columns,                         **
**                      char** select_tables, char** aliases,                **
**                      char** select_columns,                               **
**                      char* where_clause);                                 */

int db_insert_select();


/* db_update: Aktualisieren von Werten in eine Tabelle                       **
**                                                                           **
** int db_update(char* connection_name,                                      **
**               char* table, char** columns, char** values,                 **
**               char** select_tables, char** aliases,                       **
**               char* where_clause);                                        */

int db_update();


/* db_delete: Loeschen von Saetzen aus einer Tabelle                         **
**                                                                           **
** int db_delete(char* connection_name,                                      **
**               char* table,                                                **
**               char** select_tables, char** aliases,                       **
**               char* where_clause);                                        */

int db_delete();
