/* Name: llist.m                                                             **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klassen LinkedList und LinkedListElement,    **
**           mit denen sich einfach verkettete Liste bilden lassen           **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 19.10.1995                                                      **
** Zuletzt bearbeitet: 08.10.1997                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "llist.h"

#include "error.h"
#include "constant.h"

/* Klasse: LinkedList                                                        */

@implementation LinkedList

+ new
{
  return [[self alloc] init];
}

+ alloc
{
  LinkedList *newObj;
  newObj=[super alloc];
  return newObj;
}

- init
{
  self=[super init];
  head=nil;
  length=0;
  lengthLimit=LLISTNOLENLIM;
  return self;
}

- append:object0
{
  long i;
  LinkedListElement *current, *newElement;
  if(lengthLimit==LLISTNOLENLIM || length<lengthLimit)
    {
      current=head;
      for(i=0;i<length-1;i++)
	current=[current getNext];
      newElement=[LinkedListElement new:object0];
      if(newElement)
	{
	  if(current)
	    [current setNext:newElement];
	  else
	    head=newElement;
	  length++;
	}
    }
  return self;
}

- insertAt:(long)pos0 object:object0
{
  long i;
  LinkedListElement *current, *newElement;
  if(pos0>=0 && pos0<=length && (lengthLimit==LLISTNOLENLIM || length<lengthLimit))
    {
      current=head;
      if(pos0==0)
	{
	  newElement=[LinkedListElement new:object0];
	  if(newElement)
	    {
	      [newElement setNext:head];
	      head=newElement;
	      length++;
	    }
	}
      else if(pos0==length)
	return [self append:object0];
      else
	{
	  for(i=0;i<pos0-1;i++)
	    current=[current getNext];
	  newElement=[LinkedListElement new:object0];
	  if(newElement)
	    {
	      [newElement setNext:[current getNext]];
	      [current setNext:newElement];
	      length++;
	    }
	}
    }
  return self;
}

- deleteAt:(long)pos0
{
  long i;
  LinkedListElement *current, *previous;
  previous=current=head;
  if(pos0>=0 && length>0 && pos0<length)
    {
      if(pos0==0)
	{
	  current=head;
	  head=[head getNext];
	  [current setNext:nil];
	  [current freeNoDestroy];
	  length--;
	}
      else
	{
	  for(i=0;i<pos0-1;i++)
	    previous=[previous getNext];
	  current=[previous getNext];
	  [previous setNext:[current getNext]];
	  [current setNext:nil];
	  [current freeNoDestroy];
	  length--;
	}
    }
  return self;
}

- deleteAndDestroyAt:(long)pos0
{
  long i;
  LinkedListElement *current, *previous;
  previous=current=head;
  if(pos0>=0 && length>0 && pos0<length)
    {
      if(pos0==0)
	{
	  current=head;
	  head=[head getNext];
	  [current setNext:nil];
	  [current free];
	  length--;
	}
      else
	{
	  for(i=0;i<pos0-1;i++)
	    previous=[previous getNext];
	  current=[previous getNext];
	  [previous setNext:[current getNext]];
	  [current setNext:nil];
	  [current free];
	  length--;
	}
    }
  return self;
}

- deleteAll:object0
{
  long i;
  LinkedListElement *current, *previous;
  previous=current=head;
  while(current)
    {
      while(current && [current getObject]!=object0)
	{
	  previous=current;
	  current=[current getNext];
	}
      if(current==head)
	{
	  head=[current getNext];
	  [current setNext:nil];
	  [current freeNoDestroy];
	  current=head;
	  length--;
	}
      else
	if(current)
	  {
	    [previous setNext:[current getNext]];
	    [current setNext:nil];
	    [current freeNoDestroy];
	    current=[previous getNext];
	    length--;
	  }
    }
  return self;
}

- deleteFirst:object0
{
  long i;
  LinkedListElement *current, *previous;
  previous=current=head;
  if([current getObject]==object0)
    {
      head=[current getNext];
      [current setNext:nil];
      [current freeNoDestroy];
      length--;
    }
  else
    {
      while(current && [current getObject]!=object0)
	{
	  previous=current;
	  current=[current getNext];
	}
      if(current)
	{
	  [previous setNext:[current getNext]];
	  [current setNext:nil];
	  [current freeNoDestroy];
	  length--;
	}
    }
  return self;
}

- empty
{
  [head freeNoDestroy];
  head=nil;
  length=0;
  return self;
}

- emptyAndDestroy
{
  [head free];
  head=nil;
  length=0;
  return self;
}

- setLengthLimit:(long)lim0
{
  if(lim0>0)
    lengthLimit=lim0;
  return self;
}

- (LinkedListElement*)getHead
{
  return head;
}

- getObjectAt:(long)pos0
{
  long i;
  LinkedListElement *current;
  current=head;
  for(i=0;i<pos0;i++)
    current=[current getNext];
  return [current getObject];
}

- (long)getFirstPosOf:object0
{
  long returnPos;
  LinkedListElement *current;
  current=head;
  returnPos=0;
  while(current && [current getObject]!=object0)
    {
      returnPos++;
      current=[current getNext];
    }
  if(!current)
    return RETURNERROR;
  return returnPos;
}

- (BOOL)isEmpty
{
  if(length)
    return NO;
  else 
    return YES;
}

- (long)getLength
{
  return length;
}

- (long)getLengthLimit
{
  return lengthLimit;
}

- freeNoDestroy
{
  [head freeNoDestroy];
  return [super free];
}

- free
{
  [head free];
  return [super free];
}

@end

/* Klasse: LinkedListElement                                                 */

@implementation LinkedListElement

+ new:object0
{
  return [[self alloc] init:object0];
}

+ alloc
{
  LinkedListElement *newObj;
  newObj=[super alloc];
  return newObj;
}

- init:object0
{
  self=[super init];
  object=object0;
  next=nil;
  return self;
}

- setNext:(LinkedListElement*)element0
{
  if([element0 isKindOf:[LinkedListElement class]] || element0==nil)
    next=element0;
  return self;
}

- getObject
{
  return object;
}

- getNext
{
  return next;
}

- freeNoDestroy
{
  object=nil;
  [next freeNoDestroy];
  return [super free];
}

- free
{
  [object free];
  [next free];
  return [super free];
}

@end
