/* Name: lpolymer.m                                                          **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse LinearPolymer, die beschreibende      **
**           Information ueber Lineare Polymere enthaelt                     **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 24.07.1995                                                      **
** Zuletzt bearbeitet: 20.05.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "lpolymer.h"

#include "nlet.h"
#include "error.h"
#include "constant.h"

#include <string.h>
#include <stdlib.h>

/* Klasse: LinearPolymer                                                     */

@implementation LinearPolymer

+ new:nlet0 source:(char*)source0 localID:(char*)localID0 primAccession:(char*)primAccession0
{
  return [[self alloc] init:nlet0 source:source0 localID:localID0 primAccession:primAccession0];
}

+ new:nlet0
{
  return [[self alloc] init:nlet0];
}

+ alloc
{
  LinearPolymer *newObj;
  newObj=[super alloc];
  newObj->source=(char*)malloc(MINSTRINGLEN);
  newObj->localID=(char*)malloc(MINSTRINGLEN);
  newObj->primAccession=(char*)malloc(MINSTRINGLEN);
  newObj->description=(char*)malloc(DEFSTRINGLEN);
  if(!newObj->source || !newObj->localID || !newObj->primAccession || !newObj->description)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

- init:nlet0 source:(char*)source0 localID:(char*)localID0 primAccession:(char*)primAccession0
{
  self=[super init];
  source=(char*)realloc(source,strlen(source0)*sizeof(char));
  localID=(char*)realloc(localID,strlen(localID0)*sizeof(char));
  primAccession=(char*)realloc(primAccession,strlen(primAccession0)*sizeof(char));
  description=(char*)realloc(description,strlen("DEFLPOLYMERDESCRIPTION")*sizeof(char));
  if([nlet0 isKindOf:[NLet class]])
    nlet=[NLet new:[nlet0 getSequence]];
  else
    nlet=[NLet new:DEFNLETSEQUENCE];
  if(!source || !localID || !primAccession || !description)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  else
    {
      strcpy(source,source0);
      strcpy(localID,localID0);
      strcpy(primAccession,primAccession0);
      strcpy(description,DEFLPOLYMERDESCRIPTION);
    }
  return self;
}

- init:nlet0
{
  return [self init:nlet0 source:DEFLPOLYMERSOURCE localID:DEFLPOLYMERLOCALID primAccession:DEFLPOLYMERPRIMACCESSION];
}

- setSource:(char*)source0
{
  if(!strcmp(source,DEFLPOLYMERSOURCE))
    {
      source=(char*)realloc(source,strlen(source0)*sizeof(char));
      strcpy(source,source0);
    }
  return self;
}

- setLocalID:(char*)localID0
{
  if(!strcmp(localID,DEFLPOLYMERLOCALID))
    {
      localID=(char*)realloc(localID,strlen(localID0)*sizeof(char));
      strcpy(localID,localID0);
    }
  return self;
}

- setPrimAccession:(char*)primAccession0
{
  if(!strcmp(primAccession,DEFLPOLYMERPRIMACCESSION))
    {
      primAccession=(char*)realloc(primAccession,strlen(primAccession0)*sizeof(char));
      strcpy(primAccession,primAccession0);
    }
  return self;
}

- setDescription:(char*)description0
{
  description=(char*)realloc(description,strlen(description0)*sizeof(char));
  strcpy(description,description0);
  return self;
}  

- setSequence:(char*)sequence0
{
  [nlet setSequence:sequence0];
  return self;
}

- (char*)getSource
{
  char *returnSource;
  returnSource=(char*)malloc(strlen(source)*sizeof(char));
  if(!returnSource)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  strcpy(returnSource,source);
  return returnSource;
}

- (char*)getLocalID
{
  char *returnID;
  returnID=(char*)malloc(strlen(localID)*sizeof(char));
  if(!returnID)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  strcpy(returnID,localID);
  return returnID;
}

- (char*)getPrimAccession
{
  char *returnAccession;
  returnAccession=(char*)malloc(strlen(primAccession)*sizeof(char));
  if(!returnAccession)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  strcpy(returnAccession,primAccession);
  return returnAccession;
}

- (char*)getDescription
{
  char *returnDescription;
  returnDescription=(char*)malloc(strlen(description)*sizeof(char));
  if(!returnDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  strcpy(returnDescription,description);
  return returnDescription;
}

- getNLet
{
  return [nlet deepCopy];
}

- (char*)getSequence
{
  return [nlet getSequence];
}

- (char*)getSequenceFrom:(long)i0 to:(long)j0
{
  return [nlet getSequenceFrom:i0 to:j0];
}

- (char)getSequenceAt:(long)pos0
{
  return [nlet getSequenceAt:pos0];
}

- (long)getLength
{
  return [nlet getLength];
}

- (long*)getComposition
{
  return [nlet getComposition];
}

- (long)getCompositionFor:(char)char0
{
  return [nlet getCompositionFor:char0];
}

- free
{
  free(source);
  free(localID);
  free(primAccession);
  free(description);
  [nlet free];
  return [super free];
}

@end
