/* Name: matrint.m                                                           **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse MatrixInteger, die eine Matrix von    **
**           ganzen Zahlen bereitstellt                                      **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 27.02.1996                                                      **
** Zuletzt bearbeitet: 20.05.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "matrint.h"

#include "error.h"
#include "constant.h"

#include <string.h>
#include <stdlib.h>

/* Klasse: MatrixInteger                                                     */

@implementation MatrixInteger

+ new:(int**)matrix0 rows:(int)rows0 columns:(int)cols0;
{
  return [[self alloc] init:matrix0 rows:rows0 columns:cols0];
}

+ alloc
{
  MatrixInteger *newObj;
  int i;
  newObj=[super alloc];
  newObj->description=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  newObj->matrix=(int**)malloc(MAXMATRIXROWS*sizeof(int*));
  for(i=0;i<MAXMATRIXROWS;i++)
    newObj->matrix[i]=(int*)malloc(MINMATRIXCOLS*sizeof(int));
  if(!newObj->description || !newObj->matrix[MAXMATRIXROWS-1])
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

- init:(int**)matrix0 rows:(int)rows0 columns:(int)cols0;
{
  int i, j;
  self=[super init];
  rows=rows0;
  columns=cols0;
  if(rows<0 || columns<0)
    {
      [ErrorHandler error:ERROBJINIT];
      return self;
    }
  description=(char*)realloc(description,strlen(DEFMATRIXDESCRIPTION)*sizeof(char));
  matrix=(int**)realloc(matrix,rows*sizeof(int*));
  for(i=0;i<rows;i++)
    matrix[i]=(int*)realloc(matrix[i],columns*sizeof(int));
  if(!description || !matrix[rows-1])
    [ErrorHandler error:ERRMEMNOTENOUGH];
  else
    {
      strcpy(description,DEFMATRIXDESCRIPTION);
      for(i=0;i<rows;i++)
	for(j=0;j<columns;j++)
	  if(matrix[i]+j)
	    matrix[i][j]=matrix0[i][j];
	  else
	    matrix[i][j]=DEFINT;
    }
  return self;
}

- setDescription:(char*)description0
{
  description=realloc(description,strlen(description0)*sizeof(char));
  strcpy(description, description0);
  return self;
}

- setElement:(int)value0 row:(int)row0 column:(int)col0;
{
  if(row0>=0 && row0<rows && col0>=0 && col0<columns)
    matrix[row0][col0]=value0;
  return self;
}

- (char*)getDescription;
{
  char *returnDescription;
  returnDescription=(char*)malloc(strlen(description)*sizeof(char));
  if(!returnDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return DEFPOINTER;
    }
  strcpy(returnDescription,description);
  return returnDescription;
}

- (int)getRows
{
  return rows;
}

- (int)getColumns
{
  return columns;
}

- (int)getElementAtRow:(int)row0 column:(int)col0;
{
  if(row0>=0 && row0<rows && col0>=0 && col0<columns)
    return matrix[row0][col0];
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return RETURNERROR;
    }
  return RETURNERROR;
}

- free
{
  int i;
  free(description);
  for(i=0;i<rows;i++)
    free(matrix[i]);
  free(matrix);
  return [super free];
}

@end
