/* Name: nlet.m                                                              **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse NLet, die einen Sequenzabschnitt      **
**           beschreibt                                                      **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 23.07.1995                                                      **
** Zuletzt bearbeitet: 16.08.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "nlet.h"

#include "error.h"
#include "constant.h"

#include <string.h>
#include <stdlib.h>

/* Klasse: NLet                                                              */

@implementation NLet

+ new:(char*)seq0
{
  return [[self alloc] init:seq0];
}

+ alloc
{
  NLet *newObj;
  newObj=[super alloc];
  newObj->sequence=(char*)malloc(MINSEQUENCELEN*sizeof(char));
  newObj->composition=(long*)malloc(SIZEASCII*sizeof(long));
  if(!newObj->sequence || !newObj->composition)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

+ (NLet*)reverse:(NLet*)nlet0
{
  return [self reverse:nlet0 from:0 to:[nlet0 getLength]-1];
}

+ (NLet*)reverse:(NLet*)nlet0 from:(long)i0 to:(long)j0
{
  NLet *reversedNLet;
  long from, to;
  char *reversedSequence;
  long i, j;
  if(![nlet0 isKindOf:[NLet class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=[nlet0 getLength])
    to=[nlet0 getLength]-1;
  else
    to=j0;
  reversedSequence=(char*)malloc((to-from+2)*sizeof(char));
  if(!reversedSequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return nil;
    }
  strcpy(reversedSequence,"");
  i=to;
  j=0;
  while(i>=from)
    reversedSequence[j++]=[nlet0 getSequenceAt:i--];
  reversedSequence[j]='\0';
  reversedNLet=[NLet new:reversedSequence];
  free(reversedSequence);
  return reversedNLet;
}

- init:(char*)seq0
{
  long i;
  self=[super init];
  if(seq0)
    {
      length=strlen(seq0);
      sequence=(char*)realloc(sequence,(length+1)*sizeof(char));
      if(length && !sequence)
	[ErrorHandler error:ERRMEMNOTENOUGH];
      else
	{
	  strcpy(sequence,seq0);
	  sequence[length]='\0';
	  for(i=0;i<SIZEASCII;i++)
	    composition[i]=0;
	  for(i=0;i<length;i++)
	    composition[sequence[i]]++;
	}
    }
  else
    {
      length=DEFNLETLEN;
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
    }
  return self;
}

- setSequence:(char*)seq0
{
  long i;
  if(seq0)
    {
      length=strlen(seq0);
      sequence=realloc(sequence,(length+1)*sizeof(char));
      if(length && !sequence)
	[ErrorHandler error:ERRMEMNOTENOUGH];
      else
	{
	  strcpy(sequence,seq0);
	  sequence[length]='\0';
	  for(i=0;i<SIZEASCII;i++)
	    composition[i]=0;
	  for(i=0;i<length;i++)
	    composition[sequence[i]]++;
	}
    }
  return self;
}

- (char*)getSequence
{
  return [self getSequenceFrom:0 to:length-1];
}

- (char*)getSequenceFrom:(long)i0 to:(long)j0
{
  char *returnSequence;
  long from, to;
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=length)
    to=length-1;
  else
    to=j0;
  if(from<=to)
    {
      returnSequence=(char*)malloc((to-from+2)*sizeof(char));
      if(!returnSequence)
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return 0;
	}
      strcpy(returnSequence,"");
      strncat(returnSequence,sequence+from,to-from+1);
      returnSequence[to-from+1]='\0';
      return returnSequence;
    }
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}

- (char)getSequenceAt:(long)pos0
{
  if(pos0>=0 && pos0<length)
    return sequence[pos0];
  else
    {
      [ErrorHandler error:ERROBJGETINSTVAR];
      return 0;
    }
  return 0;
}

- (long)getLength
{
  return length;
}

- (long*)getComposition
{
  long *returnComposition;
  int i;
  returnComposition=(long*)malloc(SIZEASCII*sizeof(long));
  if(!returnComposition)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  for(i=0;i<SIZEASCII;i++)
    returnComposition[i]=composition[i];
  return returnComposition;
}

- (long)getCompositionFor:(char)char0
{
  return composition[char0];
}

- free
{
  free(sequence);
  free(composition);
  return [super free];
}

@end
