/* Name: rwheel.h                                                            **
**                                                                           **
**                                                                           **
** Funktion: Implementatiob der Klasse RouletteWheel, die eine               **
**           Rouletteradauswahl bereitstellt (Ableitung der Klasse           **
**           ArrayDouble                                                     **
**                                                                           **
** Aufruf: -- (Einbindung in ObjC-Dateien ueber #import bzw. #include)       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 15.11.1996                                                      **
** Zuletzt bearbeitet: 16.11.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "rwheel.h"

#include "error.h"
#include "constant.h"

#include <stdlib.h>
#include <time.h>

#ifndef RAND_MAX
#include <limits.h>
#define RAND_MAX LONG_MAX
#endif

/* Klasse: RouletteWheel                                                     */

@implementation RouletteWheel

+ new:(int)size0 segments:(double*)segments0
{
  return [self new:size0 array:segments0];
}

+ (void)initRandomGenerator
{
  int seed;
  seed=(int)time(NULL);
  srandom(seed);
}

- init:(int)size0 segments:(double*)segments0
{
  return [self init:size0 array:segments0];
}

- init:(int)size0 array:(double*)array0
{
  int i;
  self=[super init];
  size=size0;
  if(size<0)
    {
      [ErrorHandler error:ERROBJINIT];
      return self;
    }
  array=(double*)realloc(array,size*sizeof(double));
  if(!array)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return self;
    }
  for(i=0;i<size;i++)
    if(array0+i)
      if(array0[i]>=0.0)
	array[i]=array0[i];
      else
	array[i]=DEFRWHEELSEGMENT;
    else
      {
        [ErrorHandler error:ERROBJINIT];
        return self;
      }
  [[self class] initRandomGenerator];
  return self;
}

- (int)select
{
  double sum, *series;
  double selector;
  int selected;
  int threshold, lower, higher;
  int i;
  series=(double*)malloc(size*sizeof(double));
  if(!series)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  sum=0.0;
  for(i=0;i<size;i++)
    {
      sum+=array[i];
      series[i]=sum;
    }
  selector=((double)random()/(double)RAND_MAX)*sum;
  selected=0;
  lower=0;
  higher=size-1;
  while(lower<higher)
    {
      threshold=lower+(higher-lower)/2;
      if(selector>series[threshold])
	lower=threshold+1;
      else
	higher=threshold;
      selected=higher;
    }
  free(series);
  return selected;
}

@end
