/* Name: textexec.m                                                          **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse TextExecute, die die verfuegbaren     **
**           Unterprogramme im Textmodus ausfuehren laesst                   **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 21.01.1996                                                      **
** Zuletzt bearbeitet: 02.07.1998                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "textexec.h"

#include "align.h"
#include "dna.h"
#include "protein.h"
#include "dnanlet.h"
#include "protnlet.h"
#include "alignalg.h"
#include "matrint.h"
#include "rwheel.h"
#include "arraydbl.h"
#include "llist.h"
#include "error.h"
#include "constant.h"

#include "db_iface.h"

#include <ctype.h>
#include <string.h>
#include <stdlib.h>

extern int identityMatrix[NUMLETTER][NUMLETTER];
extern int standardDNAMatrix[NUMLETTER][NUMLETTER];
extern int pam250Matrix[NUMLETTER][NUMLETTER];
extern int propertyIdentityMatrix[NUMLETTER][NUMLETTER];
extern int anticodonMatrix[NUMLETTER][NUMLETTER];
extern int mirrorMatrix[NUMLETTER][NUMLETTER];
extern int inversComplementarityMatrix[NUMLETTER][NUMLETTER];

/* Klasse: TextExecute                                                       */

@implementation TextExecute

+ new:(char*)server0 logFile:(FILE*)logFile0
{
  return [[self alloc] init:server0 logFile:logFile0];
}

+ alloc
{
  TextExecute *newObj;
  newObj=[super alloc];
  newObj->dbServer=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  newObj->linPolymerList=[LinkedList new];
  newObj->curSearchList=[LinkedList new];
  newObj->curAlignmentList=[LinkedList new];
  if(!newObj->dbServer)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

- init:(char*)server0 logFile:(FILE*)logFile0
{
  self=[super init];
  useDB=NO;
  if(server0)
    {
      dbServer=realloc(dbServer,(strlen(server0)+1)*sizeof(char));
      if(!dbServer)
	{
	  [ErrorHandler error:ERRMEMNOTENOUGH];
	  return self;
	}
      strcpy(dbServer,server0);
      dbServer[strlen(dbServer)]='\0';
    }
  if(logFile0)
    logFile=logFile0;
  curQueryLPolymer=0;
  return self;
}

- (BOOL)getDBUsage
{
  return useDB;
}

- (char*)getDBServer
{
  char *getServer;
  getServer=(char*)malloc((strlen(dbServer)+1)*sizeof(char));
  if(!getServer)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return 0;
    }
  strcpy(getServer,dbServer);
  getServer[strlen(dbServer)]='\0';
  return getServer;
}

- (int)executeByShorthand:(char*)short0
{
  if(!strncasecmp(short0,SHORTNOACTION,DEFSHORTHANDLEN))
    return RETURNEXIT;
  if(!strncasecmp(short0,SHORTINPUTLP,DEFSHORTHANDLEN))
    return [self inputLinearPolymer:LPNOTYPE];
  if(!strncasecmp(short0,SHORTEDITSOURCE,DEFSHORTHANDLEN))
    return [self editSource];
  if(!strncasecmp(short0,SHORTEDITLOCALID,DEFSHORTHANDLEN))
    return [self editLocalID];
  if(!strncasecmp(short0,SHORTEDITDESCRIPTION,DEFSHORTHANDLEN))
    return [self editDescription];
  if(!strncasecmp(short0,SHORTGENERATERANDOM,DEFSHORTHANDLEN))
    return [self generateRandomLPolymer:LPNOTYPE];
  if(!strncasecmp(short0,SHORTLISTCOMPOSITION,DEFSHORTHANDLEN))
    return [self listComposition];
  if(!strncasecmp(short0,SHORTANTISENSE,DEFSHORTHANDLEN))
    return [self dnaAntisenseStrand];
  if(!strncasecmp(short0,SHORTTRANSLATE,DEFSHORTHANDLEN))
    return [self dnaTranslate];
  if(!strncasecmp(short0,SHORTDNAREVERSE,DEFSHORTHANDLEN))
    return [self dnaReverse];
  if(!strncasecmp(short0,SHORTTRIPLETREVERSE,DEFSHORTHANDLEN))
    return [self dnaTripletReverse];
  if(!strncasecmp(short0,SHORTMININFORMATION,DEFSHORTHANDLEN))
    return [self dnaMinimumInformation];
  if(!strncasecmp(short0,SHORTDENSITY,DEFSHORTHANDLEN))
    return [self dnaInformationDensity];
  if(!strncasecmp(short0,SHORTRETRANSLATE,DEFSHORTHANDLEN))
    return [self proteinRetranslate];
  if(!strncasecmp(short0,SHORTALIGNTWONLETS,DEFSHORTHANDLEN))
    return [self alignTwoNLets];
  if(!strncasecmp(short0,SHORTALIGNQUERYWBULK,DEFSHORTHANDLEN))
    return [self alignQueryNLetWBulk];
  if(!strncasecmp(short0,SHORTOUTPUTALIGN,DEFSHORTHANDLEN))
    return [self outputAlignments];
  if(!strncasecmp(short0,SHORTOUTPUTLP,DEFSHORTHANDLEN))
    return [self outputLinearPolymer];
  if(!strncasecmp(short0,SHORTCLOSELP,DEFSHORTHANDLEN))
    return [self closeLinearPolymer];
  if(!strncasecmp(short0,SHORTLISTLP,DEFSHORTHANDLEN))
    return [self listOpenLPolymers];
  if(!strncasecmp(short0,SHORTUNIXSH,DEFSHORTHANDLEN))
    return [self openUNIXShellCommand];
  if(!strncasecmp(short0,SHORTDBCONNECT,DEFSHORTHANDLEN))
    return [self dbConnect];
  if(!strncasecmp(short0,SHORTDBDISCONNECT,DEFSHORTHANDLEN))
    return [self dbDisconnect];
  if(!strncasecmp(short0,SHORTEXIT,DEFSHORTHANDLEN))
    return RETURNEXIT;
  return RETURNERROR;
}

- (int)chooseLinearPolymer:(int)type0
{
  int lPolymerNum;
  char option;
  option=0;
  while(option!='I' && option!='E' && option!='R')
    {
      option=readCharWCommandDefault(stdin,"Use linear polymer from (I)nternal list or (E)xternal source or generate a (R)andom polymer?",'I');
      option=toupper(option);
    }
  lPolymerNum=0;
  switch(option)
    {
    case 'I':
      lPolymerNum=[self chooseLPolymerFromList];
      break;
    case 'E':
      lPolymerNum=[self inputLinearPolymer:type0];
      break;
    case 'R':
      lPolymerNum=[self generateRandomLPolymer:type0];
      break;
    default:
      break;
    }
  return lPolymerNum;
}

- (int)chooseLPolymerFromList
{
  int lPolymerNum;
  lPolymerNum=readIntWCommand(stdin,"Please enter the linear polymer number");
  if(lPolymerNum<=0 || lPolymerNum>[linPolymerList getLength])
    {  
      [ErrorHandler error:ERROBJNOTPRES];
      return RETURNERROR;
    }
  return lPolymerNum-1;
}

- (int)listOpenLPolymers
{
  id current, curLPolymer;
  int num;
  current=[linPolymerList getHead];
  if(!current)
      printf("\nNo linear polymers open!\n");
  else
    {
      num=0;
      printf("\nList of open linear polymers\n");
      printf("\nNo  Type     Source      Local ID    Description\n");
      /* printf("12LL1234567LL1234567890LL1234567890LLDDDD...\n"); */
      printf("\n");
      while(current)
	{
	  curLPolymer=[current getObject];
	  printf("%.2d",++num);
	  printf("  ");
	  if([curLPolymer isKindOf:[DNA class]])
	    printf("DNA    ");
	  else 
	    if([curLPolymer isKindOf:[Protein class]])
	      printf("Protein");
	    else
	      printf("Unknown");
	  printf("  ");
	  printf("%-10.10s",[curLPolymer getSource]);
	  printf("  ");
	  printf("%-10.10s",[curLPolymer getLocalID]);
	  printf("  ");
	  printf("%.42s\n",[curLPolymer getDescription]);
	  current=[current getNext];
	}
    }
  printf("\n");
  readCharWCommand(stdin,"Press <RETURN> to continue...");
  return RETURNEXIT;
}

- (char*)buildSelectStatement
{
  char choice;
  char *databaseIDChoice, *localIDChoice, *primAccessionChoice, *speciesChoice, *classChoice, *keywordChoice, *featureChoice;
  char *curCriteria;
  char *sqlCommand, *from, *where, *chosenWhere;
  int i, j;
  databaseIDChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  localIDChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  primAccessionChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  speciesChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  classChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  keywordChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  featureChoice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  curCriteria=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  sqlCommand=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  from=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  where=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  chosenWhere=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  if(!databaseIDChoice || !localIDChoice || !primAccessionChoice || !speciesChoice || !classChoice || !keywordChoice || !featureChoice || !curCriteria || !sqlCommand || !from || !where)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(databaseIDChoice);
      free(localIDChoice);
      free(primAccessionChoice);
      free(speciesChoice);
      free(classChoice);
      free(keywordChoice);
      free(featureChoice);
      free(curCriteria);
      free(sqlCommand);
      free(from);
      free(where);
      free(chosenWhere);
      return "";
    }
  strcpy(sqlCommand,"SELECT LINEAR_POLYMER.DATABASE_ID, LOCAL_ID, LINEAR_POLYMER.PRIMARY_ACCESSION, DESCRIPTION, SEQUENCE ");
  strcpy(from,"FROM LINEAR_POLYMER, SEQUENCED_TO ");
  strcpy(where,"WHERE LINEAR_POLYMER.DATABASE_ID=SEQUENCED_TO.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=SEQUENCED_TO.PRIMARY_ACCESSION");
  strcpy(databaseIDChoice,"");
  strcpy(localIDChoice,"");
  strcpy(primAccessionChoice,"");
  strcpy(speciesChoice,"");
  strcpy(classChoice,"");
  strcpy(keywordChoice,"");
  strcpy(featureChoice,"");
  strcpy(chosenWhere," AND (");
  do
    {
      printf("Enter search criteria:\n");
      printf("1   Database ID: %.42s\n",databaseIDChoice);
      printf("2   Local ID: %.42s\n",localIDChoice);
      printf("3   Primary accession: %.42s\n",primAccessionChoice);
      printf("4   Species: %.42s\n",speciesChoice);
      printf("5   Species class: %.42s\n",classChoice);
      printf("6   Keyword: %.42s\n",keywordChoice);
      printf("7   Feature key: %.42s\n",featureChoice);
      printf("8   Enter complete SELECT statement\n");
      printf("9   Discard current criteria\n");
      printf("0   Done\n");
      choice=readCharWCommandDefault(stdin,"Please specify the attribute you wish to restrict",'0');
      switch(choice)
	{
	case '1':
	  readStringWCommandHead(databaseIDChoice,stdin,"Please enter the database ID's (seperated by comma)",databaseIDChoice);
	  break;
	case '2':
	  readStringWCommandHead(localIDChoice,stdin,"Please enter the local ID's (seperated by comma)",localIDChoice);
	  break;
	case '3':
	  readStringWCommandHead(primAccessionChoice,stdin,"Please enter the primary accession numbers (seperated by comma)",primAccessionChoice);
	  break;
	case '4':
	  readStringWCommandHead(speciesChoice,stdin,"Please enter the species (seperated by comma)",speciesChoice);
	  break;
	case '5':
	  readStringWCommandHead(classChoice,stdin,"Please enter the species classes (seperated by comma)",classChoice);
	  break;
	case '6':
	  readStringWCommandHead(keywordChoice,stdin,"Please enter the keywords (seperated by comma)",keywordChoice);
	  break;
	case '7':
	  readStringWCommandHead(featureChoice,stdin,"Please enter the feature keys (seperated by comma)",featureChoice);
	  break;
	case '8':
	  strcpy(databaseIDChoice,"");
	  strcpy(localIDChoice,"");
	  strcpy(primAccessionChoice,"");
	  strcpy(speciesChoice,"");
	  strcpy(classChoice,"");
	  strcpy(keywordChoice,"");
	  strcpy(featureChoice,"");
	  strcat(sqlCommand,from);
	  strcpy(from,"");
	  strcpy(where,"");
	  readStringWCommandHead(sqlCommand,stdin,"Please enter the SELECT statement",sqlCommand);
	  break;
	case '9':
	  strcpy(sqlCommand,"SELECT LINEAR_POLYMER.DATABASE_ID, LOCAL_ID, LINEAR_POLYMER.PRIMARY_ACCESSION, DESCRIPTION, SEQUENCE ");
	  strcpy(from,"FROM LINEAR_POLYMER, SEQUENCED_TO ");
	  strcpy(where,"WHERE LINEAR_POLYMER.DATABASE_ID=SEQUENCED_TO.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=SEQUENCED_TO.PRIMARY_ACCESSION AND (");
	  strcpy(databaseIDChoice,"");
	  strcpy(localIDChoice,"");
	  strcpy(primAccessionChoice,"");
	  strcpy(speciesChoice,"");
	  strcpy(classChoice,"");
	  strcpy(keywordChoice,"");
	  strcpy(featureChoice,"");
	  strcpy(from,"FROM LINEAR_POLYMER, SEQUENCED_TO ");
	  strcpy(where,"WHERE LINEAR_POLYMER.DATABASE_ID=SEQUENCED_TO.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=SEQUENCED_TO.PRIMARY_ACCESSION");
	  break;
	default:
	  break;
	}
    }
  while(choice!='0');
  if(strcmp(databaseIDChoice,""))
    {
      i=0;
      strcat(chosenWhere,"(");
      while(databaseIDChoice[i])
	{
	  while(databaseIDChoice[i]==' ')
	    i++;
	  j=0;
	  while(databaseIDChoice[i] && databaseIDChoice[i++]!=',')
	    curCriteria[j++]=databaseIDChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"LINEAR_POLYMER.DATABASE_ID='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,")");
    }
  if(strcmp(localIDChoice,""))
    {
      if(strcmp(chosenWhere," AND ("))
	strcat(chosenWhere," AND ");
      i=0;
      strcat(chosenWhere,"(");
      while(localIDChoice[i])
	{
	  while(localIDChoice[i]==' ')
	    i++;
	  j=0;
	  while(localIDChoice[i] && localIDChoice[i++]!=',')
	    curCriteria[j++]=localIDChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"LOCAL_ID='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,")");
    }
  if(strcmp(primAccessionChoice,""))
    {
      if(strcmp(chosenWhere," AND ("))
	strcat(chosenWhere," OR ");
      i=0;
      strcat(chosenWhere,"(");
      while(primAccessionChoice[i])
	{
	  while(primAccessionChoice[i]==' ')
	    i++;
	  j=0;
	  while(primAccessionChoice[i] && primAccessionChoice[i++]!=',')
	    curCriteria[j++]=primAccessionChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"LINEAR_POLYMER.PRIMARY_ACCESSION='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,")");
    }
  if(strcmp(speciesChoice,""))
    {
      from[strlen(from)-1]=',';
      from[strlen(from)]='\0';
      strcat(from," OCCURS_IN, COMMON_NAMES ");
      if(strcmp(chosenWhere," AND ("))
	strcat(chosenWhere," OR ");
      i=0;
      strcat(chosenWhere,"(LINEAR_POLYMER.DATABASE_ID=OCCURS_IN.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=OCCURS_IN.PRIMARY_ACCESSION AND OCCURS_IN.SPECIES_NAME=COMMON_NAMES.SPECIES_NAME AND (");
      while(speciesChoice[i])
	{
	  while(speciesChoice[i]==' ')
	    i++;
	  j=0;
	  while(speciesChoice[i] && speciesChoice[i++]!=',')
	    curCriteria[j++]=speciesChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"OCCURS_IN.SPECIES_NAME='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR COMMON_NAMES.COMMON_NAME='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,"))");
    }
  if(strcmp(classChoice,""))
    {
      from[strlen(from)-1]=',';
      from[strlen(from)]='\0';
      strcat(from," OCCURS_IN, CLASSIFIED_BY ");
      if(strcmp(chosenWhere," AND ("))
	strcat(chosenWhere," OR ");
      i=0;
      strcat(chosenWhere,"(LINEAR_POLYMER.DATABASE_ID=OCCURS_IN.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=OCCURS_IN.PRIMARY_ACCESSION AND OCCURS_IN.SPECIES_NAME=CLASSIFIED_BY.SPECIES_NAME AND (");
      while(classChoice[i])
	{
	  while(classChoice[i]==' ')
	    i++;
	  j=0;
	  while(classChoice[i] && classChoice[i++]!=',')
	    curCriteria[j++]=classChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"CLASSIFIED_BY.CLASS_NAME='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,"))");
    }
  if(strcmp(keywordChoice,""))
    {
      from[strlen(from)-1]=',';
      from[strlen(from)]='\0';
      strcat(from," IDENTIFIED_BY, SYNONYM_FOR ");
      if(strcmp(chosenWhere," AND ("))
	strcat(chosenWhere," OR ");
      i=0;
      strcat(chosenWhere,"(LINEAR_POLYMER.DATABASE_ID=IDENTIFIED_BY.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=IDENTIFIED_BY.PRIMARY_ACCESSION AND IDENTIFIED_BY.REF_KEYWORD_NAME=SYNONYM_FOR.REF_KEYWORD_NAME AND (");
      while(keywordChoice[i])
	{
	  while(keywordChoice[i]==' ')
	    i++;
	  j=0;
	  while(keywordChoice[i] && keywordChoice[i++]!=',')
	    curCriteria[j++]=keywordChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"SYNONYM_FOR.KEYWORD_NAME='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,"))");
    }
  if(strcmp(featureChoice,""))
    {
      from[strlen(from)-1]=',';
      from[strlen(from)]='\0';
      strcat(from," FEATURED_BY ");
      if(strcmp(chosenWhere," AND ("))
	strcat(chosenWhere," OR ");
      i=0;
      strcat(chosenWhere,"(LINEAR_POLYMER.DATABASE_ID=FEATURED_BY.DATABASE_ID AND LINEAR_POLYMER.PRIMARY_ACCESSION=FEATURED_BY.PRIMARY_ACCESSION AND (");
      while(featureChoice[i])
	{
	  while(featureChoice[i]==' ')
	    i++;
	  j=0;
	  while(featureChoice[i] && featureChoice[i++]!=',')
	    curCriteria[j++]=featureChoice[i-1];
	  curCriteria[j]='\0';
	  strcat(chosenWhere,"FEATURED_BY.FEATURE_KEY='");
	  strcat(chosenWhere,curCriteria);
	  strcat(chosenWhere,"' OR ");
	}
      chosenWhere[strlen(chosenWhere)-4]='\0';
      strcat(chosenWhere,"))");
    }
  chosenWhere[strlen(chosenWhere)]='\0';
  strcat(chosenWhere,")");
  if(!strcmp(chosenWhere," AND ()"))
     strcpy(chosenWhere,"");
  strcat(where,chosenWhere);
  strcat(sqlCommand,from);
  strcat(sqlCommand,where);
  strcat(sqlCommand,";");
  free(databaseIDChoice);
  free(localIDChoice);
  free(primAccessionChoice);
  free(speciesChoice);
  free(classChoice);
  free(keywordChoice);
  free(featureChoice);
  free(curCriteria);
  free(from);
  free(where);
  free(chosenWhere);
  /* printf("Selected SQL statement: %s\n",sqlCommand); */
  return sqlCommand;
}

- (int)inputLinearPolymer:(int)type0
{
  char optSource;
  if(useDB)
    while(optSource!='D' && optSource!='F' && optSource!='M')
      {
	optSource=readCharWCommandDefault(stdin,"Input a linear polymer from (D)atabase, from (F)ile or (M)anually?",'D');
	optSource=toupper(optSource);
      }
  else
    while(optSource!='F' && optSource!='M')
      {
	optSource=readCharWCommandDefault(stdin,"Input a linear polymer from (F)ile or (M)anually?",'F');
	optSource=toupper(optSource);
      }
  switch(optSource)
    {
    case 'D':
      return [self inputLPolymerFromDB:type0];
      break;
    case 'F':
      return [self inputLPolymerFromFile:type0];
      break;
    case 'M':
      return [self inputLPolymerManually:type0];
      break;
    default:
      break;
    }
  return RETURNERROR;
}

- (int)generateRandomLPolymer:(int)type0
{
  char optLPType, optComposition;
  RouletteWheel *roulWheel;
  double *composition;
  LinearPolymer *newLPolymer, *compositionLPolymer;
  NLet *newNLet;
  int type;
  char *sequence;
  long length;
  int num;
  long i;
  composition=(double*)malloc(NUMLETTER*sizeof(double));
  sequence=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  if(!composition || !sequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(composition);
      free(sequence);
      return RETURNERROR;
    }
  if(type0!=LPDNATYPE && type0!=LPPROTEINTYPE)
    {
      while(optLPType!='D' && optLPType!='P')
	{
	  optLPType=readCharWCommand(stdin,"Generate a (D)NA or a (P)rotein?");
	  optLPType=toupper(optLPType);
	}
      if(optLPType=='D')
	type=LPDNATYPE;
      else if(optLPType=='P')
	type=LPPROTEINTYPE;
    }
  else
    type=type0;
  length=readLongWCommand(stdin,"Please enter the desired length of the sequence");
  while(optComposition!='E' && optComposition!='P')
    {
      optComposition=readCharWCommand(stdin,"Compose sequence (E)qually distributed or use composition of another (P)olymer?");
      optComposition=toupper(optComposition);
    }
  if(optComposition=='P')
    {
      num=[self chooseLinearPolymer:type];
      if(num==RETURNERROR)
	{
	  free(composition);
	  free(sequence);
	  return RETURNERROR;
	}
      compositionLPolymer=[linPolymerList getObjectAt:num];
    }
  if(type==LPDNATYPE)
    {
      if(optComposition=='E')
	for(i=0;i<NUMLETTER;i++)
	  if([DNANLet isDNANucleotide:ASCIIa+i])
	    composition[i]=1.0;
	  else
	    composition[i]=0.0;
      else if(optComposition=='P')
	for(i=0;i<NUMLETTER;i++)
	  composition[i]=(double)[compositionLPolymer getCompositionFor:ASCIIa+i];
      roulWheel=[RouletteWheel new:NUMLETTER segments:composition];
      for(i=0;i<length;i++)
	sequence[i]=ASCIIa+[roulWheel select];
      sequence[length]='\0';
      newNLet=[DNANLet new:sequence];
      newLPolymer=[DNA new:newNLet];
    }
  else if(type==LPPROTEINTYPE)
    {
      if(optComposition=='E')
	for(i=0;i<NUMLETTER;i++)
	  if([ProteinNLet isAminoAcid:ASCIIA+i])
	    composition[i]=1.0;
	  else
	    composition[i]=0.0;
      else if(optComposition=='P')
	for(i=0;i<NUMLETTER;i++)
	  composition[i]=(double)[compositionLPolymer getCompositionFor:ASCIIA+i];
      roulWheel=[RouletteWheel new:NUMLETTER segments:composition];
      for(i=0;i<length;i++)
	sequence[i]=ASCIIA+[roulWheel select];
      sequence[length]='\0';
      newNLet=[ProteinNLet new:sequence];
      newLPolymer=[Protein new:newNLet];
    }
  if(newLPolymer)
    {
      [newLPolymer setSource:"Random"];
      [newLPolymer setDescription:"Randomly generated linear polymer"];
      [linPolymerList append:newLPolymer];
    }
  else
    {
      [ErrorHandler error:ERROBJCREATE];
      [roulWheel free];
      free(composition);
      free(sequence);
      return RETURNERROR;
    }
  [newNLet free];
  [roulWheel free];
  free(composition);
  free(sequence);
  return [linPolymerList getFirstPosOf:newLPolymer];
}

- (int)inputLPolymerFromDB:(int)type0
{
  char optLPType;
  LinearPolymer *newLPolymer;
  NLet *newNLet;
  int type;
  char **fetch;
  char *sqlCommand;
  int i;
  fetch=(char**)malloc(5*sizeof(char*));
  fetch[0]=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  fetch[1]=(char*)malloc(MINSTRINGLEN*sizeof(char));
  fetch[2]=(char*)malloc(MINSTRINGLEN*sizeof(char));
  fetch[3]=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  fetch[4]=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  sqlCommand=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  if(!fetch[4] || !sqlCommand)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(fetch[0]);
      free(fetch[1]);
      free(fetch[2]);
      free(fetch[3]);
      free(fetch[4]);
      free(fetch);
      free(sqlCommand);
      return 0;
    }
  strcpy(fetch[0],"");
  strcpy(fetch[1],"");
  strcpy(fetch[2],"");
  strcpy(fetch[3],"");
  strcpy(fetch[4],"");
  if(type0!=LPDNATYPE && type0!=LPPROTEINTYPE)
    {
      while(optLPType!='D' && optLPType!='P')
	{
	  optLPType=readCharWCommand(stdin,"Input a (D)NA or a (P)rotein?");
	  optLPType=toupper(optLPType);
	}
      if(optLPType=='D')
	type=LPDNATYPE;
      else if(optLPType=='P')
	type=LPPROTEINTYPE;
    }
  else
    type=type0;
  strcpy(sqlCommand,[self buildSelectStatement]);
  db_declare_cursor(DEFDBCONNECTION,1,sqlCommand);
  db_open_cursor(DEFDBCONNECTION,1);
  if(db_fetch(DEFDBCONNECTION,1,5,fetch)==RETURNEXIT)
    if(type==LPDNATYPE)
      {
	newNLet=[DNANLet new:fetch[4]];
	newLPolymer=[DNA new:newNLet source:fetch[0] localID:fetch[1] primAccession:fetch[2]];
      }
    else if(type==LPPROTEINTYPE)
      {
	newNLet=[ProteinNLet new:fetch[4]];
	newLPolymer=[Protein new:newNLet source:fetch[0] localID:fetch[1] primAccession:fetch[2]];
      }
    else
      [ErrorHandler error:ERRDBNOFETCH];
  db_close_cursor(DEFDBCONNECTION,1);
  if(strlen(fetch[1])>0 && newLPolymer)
    {
      [newLPolymer setDescription:fetch[3]];
      [linPolymerList append:newLPolymer];
    }
  else
    {
      [ErrorHandler error:ERROBJCREATE];
      [newNLet free];
      free(fetch[0]);
      free(fetch[1]);
      free(fetch[2]);
      free(fetch[3]);
      free(fetch[4]);
      free(fetch);
      free(sqlCommand);
      return RETURNERROR;
    }
  [newNLet free];
  free(fetch[0]);
  free(fetch[1]);
  free(fetch[2]);
  free(fetch[3]);
  free(fetch[4]);
  free(fetch);
  free(sqlCommand);
  return [linPolymerList getFirstPosOf:newLPolymer];
}

- (int)inputLPolymerFromFile:(int)type0
{
  char optLPType, optFileFormat;
  char *fileName;
  FILE *file;
  LinearPolymer *newLPolymer;
  int type;
  fileName=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!fileName)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  if(type0!=LPDNATYPE && type0!=LPPROTEINTYPE)
    {
      while(optLPType!='D' && optLPType!='P')
	{
	  optLPType=readCharWCommand(stdin,"Input a (D)NA or a (P)rotein?");
	  optLPType=toupper(optLPType);
	}
      if(optLPType=='D')
	type=LPDNATYPE;
      else if(optLPType=='P')
	type=LPPROTEINTYPE;
    }
  else
    type=type0;
  readStringWCommand(fileName,stdin,"Please enter the file name");
  file=fopen(fileName,"r");
  if(!file)
    {
      [ErrorHandler error:ERRFILEOPEN];
      return RETURNERROR;
    }
  if(type==LPDNATYPE)
    {
      while(optFileFormat!='P' && optFileFormat!='E')
	{
	  optFileFormat=readCharWCommandDefault(stdin,"Input file format: (P)lain or (E)MBL?",'P');
	  optFileFormat=toupper(optFileFormat);
	}
      if(optFileFormat=='P')
	newLPolymer=[self inputDNAFile:file format:FILEPLAINDNA];
      else if(optFileFormat=='E')
	newLPolymer=[self inputDNAFile:file format:FILEEMBL];
    }
  else if(type==LPPROTEINTYPE)
    {
      while(optFileFormat!='P' && optFileFormat!='S')
	{
	  optFileFormat=readCharWCommandDefault(stdin,"Input file format: (P)lain or (S)WISS-PROT?",'P');
	  optFileFormat=toupper(optFileFormat);
	}
      if(optFileFormat=='P')
	newLPolymer=[self inputProteinFile:file format:FILEPLAINPROTEIN];
      else if(optFileFormat=='S')
	newLPolymer=[self inputProteinFile:file format:FILESWISSPROT];
    }
  else
    {
      free(fileName);
      fclose(file);
      return RETURNERROR;
    }
  if(newLPolymer)
    [linPolymerList append:newLPolymer];
  else
    {
      [ErrorHandler error:ERROBJCREATE];
      free(fileName);
      fclose(file);
      return RETURNERROR;
    }
  free(fileName);
  fclose(file);
  return [linPolymerList getFirstPosOf:newLPolymer];
}

- (int)inputLPolymerManually:(int)type0
{
  char optLPType;
  LinearPolymer *newLPolymer;
  NLet *newNLet;
  int type;
  char *sequence;
  sequence=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  if(!sequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  if(type0!=LPDNATYPE && type0!=LPPROTEINTYPE)
    {
      while(optLPType!='D' && optLPType!='P')
	{
	  optLPType=readCharWCommand(stdin,"Input a (D)NA or a (P)rotein?");
	  optLPType=toupper(optLPType);
	}
      if(optLPType=='D')
	type=LPDNATYPE;
      else if(optLPType=='P')
	type=LPPROTEINTYPE;
    }
  else
    type=type0;
  if(type==LPDNATYPE)
    {
      readStringWCommand(sequence,stdin,"Please enter the DNA sequence");
      newNLet=[DNANLet new:sequence];
      newLPolymer=[DNA new:newNLet];
    }
  else if(type==LPPROTEINTYPE)
    {
      readStringWCommand(sequence,stdin,"Please enter the protein sequence");
      newNLet=[ProteinNLet new:sequence];
      newLPolymer=[Protein new:newNLet];
    }
  if(newLPolymer)
    {
      [newLPolymer setSource:"Manual"];
      [linPolymerList append:newLPolymer];
    }
  else
    {
      [ErrorHandler error:ERROBJCREATE];
      [newNLet free];
      free(sequence);
      return RETURNERROR;
    }
  [newNLet free];
  free(sequence);
  return [linPolymerList getFirstPosOf:newLPolymer];
}

- (DNA*)inputDNAFile:(FILE*)file0 format:(int)format0
{
  DNA *newDNA;
  switch(format0)
    {
    case FILEPLAINDNA:
      newDNA=[self inputPlainDNAFile:file0];
      break;
    case FILEEMBL:
      newDNA=[self inputEMBLFile:file0];
      break;
    default:
      [ErrorHandler error:ERRFILEFORMATSUPP];
      return nil;
    }
  return newDNA;
}

- (Protein*)inputProteinFile:(FILE*)file0 format:(int)format0
{
  Protein *newProtein;
  switch(format0)
    {
    case FILEPLAINPROTEIN:
      newProtein=[self inputPlainProteinFile:file0];
      break;
    case FILESWISSPROT:
      newProtein=[self inputSWISSPROTFile:file0];
      break;
    default:
      [ErrorHandler error:ERRFILEFORMATSUPP];
      return nil;
    }
  return newProtein;
}

- (DNA*)inputPlainDNAFile:(FILE*)file0
{
  DNA *newDNA;
  DNANLet *dnaNLet;
  char *sequence;
  char nucleotide;
  long i;
  sequence=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  if(!sequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return nil;
    }
  i=0;
  while((nucleotide=fgetc(file0))!=EOF)
    if(isalpha(nucleotide))
      sequence[i++]=nucleotide;
  sequence[i]='\0';
  dnaNLet=[DNANLet new:sequence];
  newDNA=[DNA new:dnaNLet];
  [newDNA setSource:"Plain file"];
  [dnaNLet free];
  free(sequence);
  return newDNA;
}

- (DNA*)inputEMBLFile:(FILE*)file0
{
  DNA *newDNA;
  DNANLet *dnaNLet;
  char *id, *primAccession, *description, *sequence;
  char *inputLine;
  char nucleotide;
  long i, j;
  id=(char*)malloc(MINSTRINGLEN*sizeof(char));
  primAccession=(char*)malloc(MINSTRINGLEN*sizeof(char));
  description=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  sequence=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  inputLine=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!id || !primAccession || !description || !sequence || !inputLine)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(id);
      free(primAccession);
      free(description);
      free(sequence);
      free(inputLine);
      return 0;
    }
  fgets(inputLine,DEFSTRINGLEN,file0);
  if(strncmp(inputLine,"ID",2))
    {
      free(id);
      free(primAccession);
      free(description);
      free(sequence);
      free(inputLine);
      return 0;
    }
  for(i=0;i<MINSTRINGLEN && inputLine[5+i]!='\n' && inputLine[5+i]!=' ';i++)
    id[i]=inputLine[5+i];
  id[i]='\0';
  while(strncmp(inputLine,"AC",2))
    if(!fgets(inputLine,DEFSTRINGLEN,file0))
      {
	free(id);
	free(primAccession);
	free(description);
	free(sequence);
	free(inputLine);
	return 0;
      }
  for(i=0;i<MINSTRINGLEN && inputLine[5+i]!='\n' && inputLine[5+i]!=';';i++)
    primAccession[i]=inputLine[5+i];
  primAccession[i]='\0';
  while(strncmp(inputLine,"DE",2))
    if(!fgets(inputLine,DEFSTRINGLEN,file0))
      {
	free(id);
	free(primAccession);
	free(description);
	free(sequence);
	free(inputLine);
	return 0;
      }
  j=0;
  while(!strncmp(inputLine,"DE",2))
    {
      if(j>0)
	description[j++]=' ';
      for(i=0;i<DEFSTRINGLEN && inputLine[5+i]!='\n';i++)
	description[j++]=tolower(inputLine[5+i]);
      fgets(inputLine,DEFSTRINGLEN,file0);
    }
  description[j]='\0';
  while(strncmp(inputLine,"SQ",2))
    if(!fgets(inputLine,DEFSTRINGLEN,file0))
      {
	free(id);
	free(primAccession);
	free(description);
	free(sequence);
	free(inputLine);
	return 0;
      }
  j=0;
  while(fgets(inputLine,DEFSTRINGLEN,file0) && strncmp(inputLine,"//",2))
    {
      i=5;
      while((nucleotide=inputLine[i++])!='\n' && i<MAXSTRINGLEN)
	if(isalpha(nucleotide))
	  sequence[j++]=tolower(nucleotide);
    }
  sequence[j]='\0';
  if(strncmp(inputLine,"//",2))
    {
      free(id);
      free(primAccession);
      free(description);
      free(sequence);
      free(inputLine);
      return 0;
    }
  dnaNLet=[DNANLet new:sequence];
  newDNA=[DNA new:dnaNLet source:"EMBL file" localID:id primAccession:primAccession];
  [newDNA setDescription:description];
  [dnaNLet free];
  free(id);
  free(primAccession);
  free(description);
  free(sequence);
  free(inputLine);
  return newDNA;
}

- (Protein*)inputPlainProteinFile:(FILE*)file0
{
  Protein *newProtein;
  ProteinNLet *proteinNLet;
  char *sequence;
  char aminoAcid;
  long i;
  sequence=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  if(!sequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return nil;
    }
  i=0;
  while((aminoAcid=fgetc(file0))!=EOF)
    if(isalpha(aminoAcid))
      sequence[i++]=aminoAcid;
  sequence[i]='\0';
  proteinNLet=[ProteinNLet new:sequence];
  newProtein=[Protein new:proteinNLet];
  [newProtein setSource:"Plain file"];
  [proteinNLet free];
  free(sequence);
  return newProtein;
}

- (Protein*)inputSWISSPROTFile:(FILE*)file0
{
  Protein *newProtein;
  ProteinNLet *proteinNLet;
  char *id, *primAccession, *description, *sequence;
  char *inputLine;
  char aminoAcid;
  long i, j;
  id=(char*)malloc(MINSTRINGLEN*sizeof(char));
  primAccession=(char*)malloc(MINSTRINGLEN*sizeof(char));
  description=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  sequence=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  inputLine=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!id || !primAccession || !description || !sequence || !inputLine)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(id);
      free(primAccession);
      free(description);
      free(sequence);
      free(inputLine);
      return 0;
    }
  fgets(inputLine,DEFSTRINGLEN,file0);
  if(strncmp(inputLine,"ID",2))
    {
      free(id);
      free(primAccession);
      free(description);
      free(sequence);
      free(inputLine);
      return 0;
    }
  for(i=0;i<MINSTRINGLEN && inputLine[5+i]!='\n' && inputLine[5+i]!=' ';i++)
    id[i]=inputLine[5+i];
  id[i]='\0';
  while(strncmp(inputLine,"AC",2))
    if(!fgets(inputLine,DEFSTRINGLEN,file0))
      {
	free(id);
	free(primAccession);
	free(description);
	free(sequence);
	free(inputLine);
	return 0;
      }
  for(i=0;i<MINSTRINGLEN && inputLine[5+i]!='\n' && inputLine[5+i]!=';';i++)
    primAccession[i]=inputLine[5+i];
  primAccession[i]='\0';
  while(strncmp(inputLine,"DE",2))
    if(!fgets(inputLine,DEFSTRINGLEN,file0))
      {
	free(id);
	free(primAccession);
	free(description);
	free(sequence);
	free(inputLine);
	return 0;
      }
  j=0;
  while(!strncmp(inputLine,"DE",2))
    {
      if(j>0)
	description[j++]=' ';
      for(i=0;i<DEFSTRINGLEN && inputLine[5+i]!='\n';i++)
	description[j++]=tolower(inputLine[5+i]);
      fgets(inputLine,DEFSTRINGLEN,file0);
    }
  description[j]='\0';
  while(strncmp(inputLine,"SQ",2))
    if(!fgets(inputLine,DEFSTRINGLEN,file0))
      {
	free(id);
	free(primAccession);
	free(description);
	free(sequence);
	free(inputLine);
	return 0;
      }
  j=0;
  while(fgets(inputLine,DEFSTRINGLEN,file0) && strncmp(inputLine,"//",2))
    {
      i=5;
      while((aminoAcid=inputLine[i++])!='\n' && i<MAXSTRINGLEN)
	if(isalpha(aminoAcid))
	  sequence[j++]=toupper(aminoAcid);
    }
  sequence[j]='\0';
  if(strncmp(inputLine,"//",2))
    {
      free(id);
      free(primAccession);
      free(description);
      free(sequence);
      free(inputLine);
      return 0;
    }
  proteinNLet=[ProteinNLet new:sequence];
  newProtein=[Protein new:proteinNLet source:"SWISS-PROT file" localID:id primAccession:primAccession];
  [newProtein setDescription:description];
  [proteinNLet free];
  free(id);
  free(primAccession);
  free(description);
  free(sequence);
  free(inputLine);
  return newProtein;
}

- (int)editSource
{
  int num;
  LinearPolymer *curLPolymer;
  char *newSource;
  newSource=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!newSource)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPNOTYPE];
  if(num==RETURNERROR)
    {
      free(newSource);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  readStringWCommand(newSource,stdin,"Please enter the new source");
  [curLPolymer setSource:newSource];
  free(newSource);
  return RETURNEXIT;
}

- (int)editLocalID
{
  int num;
  LinearPolymer *curLPolymer;
  char *newLocalID;
  newLocalID=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!newLocalID)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPNOTYPE];
  if(num==RETURNERROR)
    {
      free(newLocalID);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  readStringWCommand(newLocalID,stdin,"Please enter the new local ID");
  [curLPolymer setLocalID:newLocalID];
  free(newLocalID);
  return RETURNEXIT;
}

- (int)editDescription
{
  int num;
  LinearPolymer *curLPolymer;
  char *newDescription;
  newDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!newDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPNOTYPE];
  if(num==RETURNERROR)
    {
      free(newDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  readStringWCommand(newDescription,stdin,"Please enter the new description");
  [curLPolymer setDescription:newDescription];
  free(newDescription);
  return RETURNEXIT;
}

- (int)listComposition
{
  int num;
  LinearPolymer *curLPolymer;
  int i;
  num=[self chooseLinearPolymer:LPNOTYPE];
  if(num==RETURNERROR)
    return RETURNERROR;
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      printf("\n");
      printf("Composition of given DNA:\n");
      printf("\n");
      for(i=0;i<SIZEASCII;i++)
	if([DNANLet isInDNAAlphabet:i] && (!isalpha(i) || islower(i)))
	  printf("%c: %8ld\n",i,[curLPolymer getCompositionFor:i]);
      printf("\n");
      readCharWCommand(stdin,"Press <RETURN> to continue...");
    }
  else if([curLPolymer isKindOf:[Protein class]])
    {
      printf("\n");
      printf("Composition of given protein:\n");
      printf("\n");
      for(i=0;i<SIZEASCII;i++)
	if([ProteinNLet isInProteinAlphabet:i] && (!isalpha(i) || isupper(i)))
	  printf("%c: %8ld\n",i,[curLPolymer getCompositionFor:i]);
      printf("\n");
      readCharWCommand(stdin,"Press <RETURN> to continue...");
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  return RETURNEXIT;
}

- (int)dnaAntisenseStrand
{
  int num;
  LinearPolymer *curLPolymer;
  DNA *antisenseLPolymer;
  DNANLet *antisenseNLet;
  long from, to;
  char *antisenseDescription;
  antisenseDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!antisenseDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPDNATYPE];
  if(num==RETURNERROR)
    {
      free(antisenseDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      from=readLongWCommandDefault(stdin,"Please enter the start point",1);
      to=readLongWCommandDefault(stdin,"Please enter the end point",[curLPolymer getLength]);
      antisenseNLet=[DNANLet antisenseStrand:[curLPolymer getNLet] from:from-1 to:to-1];
      antisenseLPolymer=[DNA new:antisenseNLet];
      [antisenseLPolymer setSource:"Antisense"];
      strcpy(antisenseDescription,"Antisense of ");
      strcat(antisenseDescription,[curLPolymer getLocalID]);
      if(from==1 && to==[curLPolymer  getLength])
	strcat(antisenseDescription," (complete)");
      else
	{
	  strcat(antisenseDescription," from ");
	  sprintf(antisenseDescription+strlen(antisenseDescription),"%ld",from);
	  strcat(antisenseDescription," to ");
	  sprintf(antisenseDescription+strlen(antisenseDescription),"%ld",to);
	}
      strcat(antisenseDescription,"");
      [antisenseLPolymer setDescription:antisenseDescription];
      [linPolymerList append:antisenseLPolymer];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [antisenseNLet free];
  free(antisenseDescription);
  return RETURNEXIT;
}

- (int)dnaTranslate
{
  int num;
  LinearPolymer *curLPolymer;
  Protein *translLPolymer;
  ProteinNLet *translNLet;
  long from, to;
  int offset;
  char *translDescription;
  translDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!translDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPDNATYPE];
  if(num==RETURNERROR)
    {
      free(translDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      from=readLongWCommandDefault(stdin,"Please enter the start point",1);
      to=readLongWCommandDefault(stdin,"Please enter the end point",[curLPolymer getLength]);
      offset=readIntWCommandDefault(stdin,"Please enter the reading frame",1);
      translNLet=[DNANLet translate:[curLPolymer getNLet] from:from-1 to:to-1 withOffset:offset-1];
      translLPolymer=[Protein new:translNLet];
      [translLPolymer setSource:"Translation"];
      strcpy(translDescription,"Translation of ");
      strcat(translDescription,[curLPolymer getLocalID]);
      if(from==1 && to==[curLPolymer  getLength])
	strcat(translDescription," (complete)");
      else
	{
	  strcat(translDescription," from ");
	  sprintf(translDescription+strlen(translDescription),"%ld",from);
	  strcat(translDescription," to ");
	  sprintf(translDescription+strlen(translDescription),"%ld",to);
	}
      strcat(translDescription,", reading frame ");
      sprintf(translDescription+strlen(translDescription),"%d",offset);
      strcat(translDescription,"");
      [translLPolymer setDescription:translDescription];
      [linPolymerList append:translLPolymer];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [translNLet free];
  free(translDescription);
  return RETURNEXIT;
}

- (int)dnaReverse
{
  int num;
  LinearPolymer *curLPolymer;
  DNA *reversedLPolymer;
  DNANLet *reversedNLet;
  long from, to;
  char *reversedDescription;
  reversedDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!reversedDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPDNATYPE];
  if(num==RETURNERROR)
    {
      free(reversedDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      from=readLongWCommandDefault(stdin,"Please enter the start point",1);
      to=readLongWCommandDefault(stdin,"Please enter the end point",[curLPolymer getLength]);
      reversedNLet=[DNANLet new:[[DNANLet reverse:[curLPolymer getNLet] from:from-1 to:to-1] getSequence]];
      reversedLPolymer=[DNA new:reversedNLet];
      [reversedLPolymer setSource:"Reverse"];
      strcpy(reversedDescription,"Reverse of ");
      strcat(reversedDescription,[curLPolymer getLocalID]);
      if(from==1 && to==[curLPolymer  getLength])
	strcat(reversedDescription," (complete)");
      else
	{
	  strcat(reversedDescription," from ");
	  sprintf(reversedDescription+strlen(reversedDescription),"%ld",from);
	  strcat(reversedDescription," to ");
	  sprintf(reversedDescription+strlen(reversedDescription),"%ld",to);
	}
      strcat(reversedDescription,"");
      [reversedLPolymer setDescription:reversedDescription];
      [linPolymerList append:reversedLPolymer];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [reversedNLet free];
  free(reversedDescription);
  return RETURNEXIT;
}

- (int)dnaTripletReverse
{
  int num;
  LinearPolymer *curLPolymer;
  DNA *reversedLPolymer;
  DNANLet *reversedNLet;
  long from, to;
  int offset;
  char *reversedDescription;
  reversedDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!reversedDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPDNATYPE];
  if(num==RETURNERROR)
    {
      free(reversedDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      from=readLongWCommandDefault(stdin,"Please enter the start point",1);
      to=readLongWCommandDefault(stdin,"Please enter the end point",[curLPolymer getLength]);
      offset=readIntWCommandDefault(stdin,"Please enter the reading frame",1);
      reversedNLet=[DNANLet tripletReverse:[curLPolymer getNLet] from:from-1 to:to-1 withOffset:offset-1];
      reversedLPolymer=[DNA new:reversedNLet];
      [reversedLPolymer setSource:"Triplet reverse"];
      strcpy(reversedDescription,"Triplet reverse of ");
      strcat(reversedDescription,[curLPolymer getLocalID]);
      if(from==1 && to==[curLPolymer  getLength])
	strcat(reversedDescription," (complete)");
      else
	{
	  strcat(reversedDescription," from ");
	  sprintf(reversedDescription+strlen(reversedDescription),"%ld",from);
	  strcat(reversedDescription," to ");
	  sprintf(reversedDescription+strlen(reversedDescription),"%ld",to);
	}
      strcat(reversedDescription,"");
      [reversedLPolymer setDescription:reversedDescription];
      [linPolymerList append:reversedLPolymer];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [reversedNLet free];
  free(reversedDescription);
  return RETURNEXIT;
}

- (int)dnaMinimumInformation
{
  int num;
  LinearPolymer *curLPolymer;
  DNA *minInformationLPolymer;
  DNANLet *minInformationNLet;
  long from, to;
  char *minInformationDescription;
  minInformationDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!minInformationDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPDNATYPE];
  if(num==RETURNERROR)
    {
      free(minInformationDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      from=readLongWCommandDefault(stdin,"Please enter the start point",1);
      to=readLongWCommandDefault(stdin,"Please enter the end point",[curLPolymer getLength]);
      minInformationNLet=[DNANLet minimumInformation:[curLPolymer getNLet] from:from-1 to:to-1];
      minInformationLPolymer=[DNA new:minInformationNLet];
      [minInformationLPolymer setSource:"Minimum information"];
      strcpy(minInformationDescription,"Minimum information of ");
      strcat(minInformationDescription,[curLPolymer getLocalID]);
      if(from==1 && to==[curLPolymer  getLength])
	strcat(minInformationDescription," (complete)");
      else
	{
	  strcat(minInformationDescription," from ");
	  sprintf(minInformationDescription+strlen(minInformationDescription),"%ld",from);
	  strcat(minInformationDescription," to ");
	  sprintf(minInformationDescription+strlen(minInformationDescription),"%ld",to);
	}
      strcat(minInformationDescription,"");
      [minInformationLPolymer setDescription:minInformationDescription];
      [linPolymerList append:minInformationLPolymer];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [minInformationNLet free];
  free(minInformationDescription);
  return RETURNEXIT;
}

- (int)dnaInformationDensity
{
  int num;
  id curLPolymer;
  num=[self chooseLinearPolymer:LPDNATYPE];
  if(num==RETURNERROR)
    return RETURNERROR;
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[DNA class]])
    {
      printf("\n");
      printf("Information density of given DNA is: %f\n",[curLPolymer getInformationDensity]);
      printf("\n");
      readCharWCommand(stdin,"Press <RETURN> to continue...");
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  return RETURNEXIT;
}

- (int)proteinRetranslate
{
  int num;
  LinearPolymer *curLPolymer;
  DNA *retranslLPolymer;
  DNANLet *retranslNLet;
  long from, to;
  char *retranslDescription;
  retranslDescription=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!retranslDescription)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  num=[self chooseLinearPolymer:LPPROTEINTYPE];
  if(num==RETURNERROR)
    {
      free(retranslDescription);
      return RETURNERROR;
    }
  curLPolymer=[linPolymerList getObjectAt:num];
  if([curLPolymer isKindOf:[Protein class]])
    {
      from=readLongWCommandDefault(stdin,"Please enter the start point",1);
      to=readLongWCommandDefault(stdin,"Please enter the end point",[curLPolymer getLength]);
      retranslNLet=[ProteinNLet retranslate:[curLPolymer getNLet] from:from-1 to:to-1];
      retranslLPolymer=[DNA new:retranslNLet];
      [retranslLPolymer setSource:"Retranslation"];
      strcpy(retranslDescription,"Retranslation of ");
      strcat(retranslDescription,[curLPolymer getLocalID]);
      if(from==1 && to==[curLPolymer  getLength])
	strcat(retranslDescription," (complete)");
      else
	{
	  strcat(retranslDescription," from ");
	  sprintf(retranslDescription+strlen(retranslDescription),"%ld",from);
	  strcat(retranslDescription," to ");
	  sprintf(retranslDescription+strlen(retranslDescription),"%ld",to);
	}
      strcat(retranslDescription,"");
      [retranslLPolymer setDescription:retranslDescription];
      [linPolymerList append:retranslLPolymer];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [retranslNLet free];
  free(retranslDescription);
  return RETURNEXIT;
}

- (int)chooseAlignmentAlgorithm
{
  int selectedAlgorithm;
  char *choice;
  choice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!choice)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  printf("Choose an alignment algorithm:\n");
  printf(" 1   Needleman and Wunsch\n");
  printf(" 2   Needleman and Wunsch (considering neighbourhood)\n");
  readStringWCommandDefault(choice,stdin,"Please enter your choice","1");
  switch(choice[0])
    {
    case '1':
      selectedAlgorithm=ALIGNALGNEEDLEWUNSCH;
      break;
    case '2':
      selectedAlgorithm=ALIGNALGNEIGHBOURNWUNSCH;
      break;
    default:
      free(choice);
      return [self chooseAlignmentAlgorithm];
    }
  free(choice);
  return selectedAlgorithm;
}

- (int)chooseAlignmentMatrix:(int)type0
{
  int selectedMatrix;
  char *choice;
  choice=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!choice)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  printf("Choose an alignment matrix:\n");
  switch(type0)
    {
    case LPDNATYPE:
      printf(" 1   Identity matrix\n");
      printf(" 2   DNA: Standard matrix\n");
      readStringWCommandDefault(choice,stdin,"Please enter your choice","2");
      switch(choice[0])
	{
	case '1':
	  selectedMatrix=ALMATRIXIDENTITY;
	  break;
	case '2':
	  selectedMatrix=ALMATRIXDNASTANDARD;
	  break;
	default:
	  free(choice);
	  return [self chooseAlignmentMatrix:type0];
	}
      break;
    case LPPROTEINTYPE:
      printf(" 1   Identity matrix\n");
      printf(" 2   Protein: PAM250 matrix\n");
      readStringWCommandDefault(choice,stdin,"Please enter your choice","2");
      switch(choice[0])
	{
	case '1':
	  selectedMatrix=ALMATRIXIDENTITY;
	  break;
	case '2':
	  selectedMatrix=ALMATRIXPAM250;
	  break;
	default:
	  free(choice);
	  return [self chooseAlignmentMatrix:type0];
	}
      break;
    default:
      printf(" 1   Identity matrix\n");
      printf(" 2   DNA: Standard matrix\n");
      printf(" 3   Protein: PAM250 matrix\n");
      readStringWCommand(choice,stdin,"Please enter your choice");
      switch(choice[0])
	{
	case '1':
	  selectedMatrix=ALMATRIXIDENTITY;
	  break;
	case '2':
	  selectedMatrix=ALMATRIXDNASTANDARD;
	  break;
	case '3':
	  selectedMatrix=ALMATRIXPAM250;
	  break;
	default:
	  free(choice);
	  return [self chooseAlignmentMatrix:type0];
	}
      break;
    }
  free(choice);
  return selectedMatrix;
}

- (int)alignTwoNLets
{
  int num1, num2;
  int algorithmSelector, matrixSelector;
  LinearPolymer *lPolymer1, *lPolymer2;
  Alignment *align;
  long from1, to1, from2, to2;
  int **matrixValues;
  MatrixInteger *matrix;
  int firstGap, restGap;
  char optOutput;
  int i, j;
  matrixValues=(int**)malloc(NUMLETTER*sizeof(int*));
  for(i=0;i<NUMLETTER;i++)
    matrixValues[i]=(int*)malloc(NUMLETTER*sizeof(int));
  if(!matrixValues[NUMLETTER-1])
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  printf("Choose first linear polymer:\n");
  num1=[self chooseLinearPolymer:LPNOTYPE];
  if(num1==RETURNERROR)
    {
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  lPolymer1=[linPolymerList getObjectAt:num1];
  from1=readLongWCommandDefault(stdin,"Please enter the start point",1);
  to1=readLongWCommandDefault(stdin,"Please enter the end point",[lPolymer1 getLength]);
  if([lPolymer1 isKindOf:[DNA class]])
    { 
      printf("Choose second linear polymer:\n");
      num2=[self chooseLinearPolymer:LPDNATYPE];
    }
  else if([lPolymer1 isKindOf:[Protein class]])
    {
      printf("Choose second linear polymer:\n");
      num2=[self chooseLinearPolymer:LPPROTEINTYPE];
    }
  else
    {
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  if(num2==RETURNERROR)
    {
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  lPolymer2=[linPolymerList getObjectAt:num2];
  from2=readLongWCommandDefault(stdin,"Please enter the start point",1);
  to2=readLongWCommandDefault(stdin,"Please enter the end point",[lPolymer2 getLength]);
  curQueryLPolymer=num1;
  [curSearchList empty];
  [curAlignmentList emptyAndDestroy];
  if([lPolymer1 class]==[lPolymer2 class])
    {
      algorithmSelector=[self chooseAlignmentAlgorithm];
      if([lPolymer1 isKindOf:[DNA class]])
	matrixSelector=[self chooseAlignmentMatrix:LPDNATYPE];
      else if([lPolymer1 isKindOf:[Protein class]])
	matrixSelector=[self chooseAlignmentMatrix:LPPROTEINTYPE];
      switch(matrixSelector)
	{
	case ALMATRIXIDENTITY:
	  for(i=0;i<NUMLETTER;i++)
	    for(j=0;j<NUMLETTER;j++)
	      matrixValues[i][j]=identityMatrix[i][j];
	  break;
	case ALMATRIXDNASTANDARD:
	  for(i=0;i<NUMLETTER;i++)
	    for(j=0;j<NUMLETTER;j++)
	      matrixValues[i][j]=standardDNAMatrix[i][j];
	  break;
	case ALMATRIXPAM250:
	  for(i=0;i<NUMLETTER;i++)
	    for(j=0;j<NUMLETTER;j++)
	      matrixValues[i][j]=pam250Matrix[i][j];
	  break;
	default:
	  [ErrorHandler error:ERROBJNOTPRES];
	  for(i=0;i<NUMLETTER;i++)
	    free(matrixValues[i]);
	  free(matrixValues);
	  return RETURNERROR;
	}
      matrix=[MatrixInteger new:matrixValues rows:NUMLETTER columns:NUMLETTER];
      firstGap=readIntWCommand(stdin,"Please enter the penalty for opening a gap");
      restGap=readIntWCommand(stdin,"Please enter the penalty for continuing a gap");
      switch(algorithmSelector)
	{
	case ALIGNALGNEEDLEWUNSCH:
	  align=[NLet needlemanWunschOf:[lPolymer1 getNLet] from:from1-1 to:to1-1 and:[lPolymer2 getNLet] from:from2-1 to:to2-1 withMatrix:matrix firstGapPenalty:firstGap restGapPenalty:restGap];
	  break;
	case ALIGNALGNEIGHBOURNWUNSCH:
	  align=[NLet neighbourhoodNeedlemanWunschOf:[lPolymer1 getNLet] from:from1-1 to:to1-1 and:[lPolymer2 getNLet] from:from2-1 to:to2-1 withMatrix:matrix firstGapPenalty:firstGap restGapPenalty:restGap];
	  break;
	default:
	  [ErrorHandler error:ERRMETHODNOTPRES];
	  for(i=0;i<NUMLETTER;i++)
	    free(matrixValues[i]);
	  free(matrixValues);
	  return RETURNERROR;
	}
      [curSearchList append:lPolymer2];
      [curAlignmentList append:align];
      optOutput=readCharWCommandDefault(stdin,"Alignment completed, output to file now?",'y');
      if(toupper(optOutput)=='Y')
	[self outputAlignments];
    }
  else
    [ErrorHandler error:ERRMETHODWRONGOBJ];
  [matrix free];
  for(i=0;i<NUMLETTER;i++)
    free(matrixValues[i]);
  free(matrixValues);
  return RETURNEXIT;
}

- (int)alignQueryNLetWBulk
{
  char optSource;
  int num;
  LinearPolymer *lPolymer1;
  long from, to;
  printf("Choose query linear polymer:\n");
  num=[self chooseLinearPolymer:LPNOTYPE];
  if(num==RETURNERROR)
    return RETURNERROR;
  lPolymer1=[linPolymerList getObjectAt:num];
  from=readLongWCommandDefault(stdin,"Please enter the start point",1);
  to=readLongWCommandDefault(stdin,"Please enter the end point",[lPolymer1 getLength]);
  if(useDB)
    while(optSource!='D' && optSource!='F')
      {
	optSource=readCharWCommandDefault(stdin,"Search over (D)atabase or (F)ile library?",'D');
	optSource=toupper(optSource);
      }
  else
    optSource='F';
  if(optSource=='D')
    return [self alignQueryNLetWBulkFromDB:lPolymer1 from:from-1 to:to-1];
  else if(optSource=='F')
    return [self alignQueryNLetWBulkFromFile:lPolymer1 from:from-1 to:to-1];
  else
    return RETURNERROR;
}

- (int)alignQueryNLetWBulkFromDB:(LinearPolymer*)lPolymer0 from:(long)i0 to:(long)j0
{
  int algorithmSelector, matrixSelector;
  id nlet2;
  LinearPolymer *lPolymer2;
  Alignment *align;
  char **fetch;
  char *sqlCommand;
  int **matrixValues;
  MatrixInteger *matrix;
  int firstGap, restGap;
  long alignListLength;
  int curQuality;
  long curLength, insertPos;
  long threshold;
  id delObject;
  char optOutput;
  long i, j;
  fetch=(char**)malloc(5*sizeof(char*));
  fetch[0]=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  fetch[1]=(char*)malloc(MINSTRINGLEN*sizeof(char));
  fetch[2]=(char*)malloc(MINSTRINGLEN*sizeof(char));
  fetch[3]=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  fetch[4]=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  matrixValues=(int**)malloc(NUMLETTER*sizeof(int*));
  sqlCommand=(char*)malloc(MAXSTRINGLEN*sizeof(char));
  for(i=0;i<NUMLETTER;i++)
    matrixValues[i]=(int*)malloc(NUMLETTER*sizeof(int));
  if(!matrixValues[NUMLETTER-1] || !fetch[4] || !sqlCommand)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      free(fetch[0]);
      free(fetch[1]);
      free(fetch[2]);
      free(fetch[3]);
      free(fetch[4]);
      free(fetch);
      free(sqlCommand);
      return RETURNERROR;
    }
  strcpy(fetch[0],"");
  strcpy(fetch[1],"");
  strcpy(fetch[2],"");
  strcpy(fetch[3],"");
  strcpy(fetch[4],"");
  if(useDB)
    {
      if(!lPolymer0)
	{
	  [ErrorHandler error:ERROBJNOTPRES];
	  for(i=0;i<NUMLETTER;i++)
	    free(matrixValues[i]);
	  free(matrixValues);
	  free(fetch[0]);
	  free(fetch[1]);
	  free(fetch[2]);
	  free(fetch[3]);
	  free(fetch[4]);
	  free(fetch);
	  free(sqlCommand);
	  return RETURNERROR;
	}
      strcpy(sqlCommand,[self buildSelectStatement]);
      algorithmSelector=[self chooseAlignmentAlgorithm];
      if([lPolymer0 isKindOf:[DNA class]])
	matrixSelector=[self chooseAlignmentMatrix:LPDNATYPE];
      else if([lPolymer0 isKindOf:[Protein class]])
	matrixSelector=[self chooseAlignmentMatrix:LPPROTEINTYPE];
      switch(matrixSelector)
	{
	case ALMATRIXIDENTITY:
	  for(i=0;i<NUMLETTER;i++)
	    for(j=0;j<NUMLETTER;j++)
	      matrixValues[i][j]=identityMatrix[i][j];
	  break;
	case ALMATRIXDNASTANDARD:
	  for(i=0;i<NUMLETTER;i++)
	    for(j=0;j<NUMLETTER;j++)
	      matrixValues[i][j]=standardDNAMatrix[i][j];
	  break;
	case ALMATRIXPAM250:
	  for(i=0;i<NUMLETTER;i++)
	    for(j=0;j<NUMLETTER;j++)
	      matrixValues[i][j]=pam250Matrix[i][j];
	  break;
	default:
	  [ErrorHandler error:ERROBJNOTPRES];
	  for(i=0;i<NUMLETTER;i++)
	    free(matrixValues[i]);
	  free(matrixValues);
	  free(fetch[0]);
	  free(fetch[1]);
	  free(fetch[2]);
	  free(fetch[3]);
	  free(fetch[4]);
	  free(fetch);
	  free(sqlCommand);
	  return RETURNERROR;
	}
      matrix=[MatrixInteger new:matrixValues rows:NUMLETTER columns:NUMLETTER];
      firstGap=readIntWCommand(stdin,"Please enter the penalty for opening a gap");
      restGap=readIntWCommand(stdin,"Please enter the penalty for continuing a gap");
      alignListLength=readLongWCommandDefault(stdin,"Please enter the number of alignments kept to be output",5000);
      fprintf(logFile,"\n");
      fprintf(logFile,"COMPUTED ALIGNMENTS");
      fprintf(logFile,"\n\n");
      fprintf(logFile,"Query linear polymer: %s %s\n",[lPolymer0 getSource],[lPolymer0 getLocalID]);
      fprintf(logFile,"Description: %.65s\n",[lPolymer0 getDescription]);
      fprintf(logFile,"Length: %ld\n",[lPolymer0 getLength]);
      fflush(logFile);
      fprintf(logFile,"\n\n");
      fprintf(logFile,"Alingnment summary:\n");
      fprintf(logFile,"\n");
      fflush(logFile);
      curQueryLPolymer=[linPolymerList getFirstPosOf:lPolymer0];
      [curSearchList empty];
      [curAlignmentList emptyAndDestroy];
      db_declare_cursor(DEFDBCONNECTION,5,sqlCommand);
      db_open_cursor(DEFDBCONNECTION,5);
      while(db_fetch(DEFDBCONNECTION,5,5,fetch)==RETURNEXIT)
	{
	  if([lPolymer0 isKindOf:[DNA class]])
	    {
	      nlet2=[DNANLet new:fetch[4]];
	      lPolymer2=[DNA new:nlet2 source:fetch[0] localID:fetch[1] primAccession:fetch[2]];
	    }
	  else if([lPolymer0 isKindOf:[Protein class]])
	    {
	      nlet2=[ProteinNLet new:fetch[4]];
	      lPolymer2=[Protein new:nlet2 source:fetch[0] localID:fetch[1] primAccession:fetch[2]];
	    }
	  else
	    {
	      nlet2=[NLet new:fetch[4]];
	      lPolymer2=[LinearPolymer new:nlet2 source:fetch[0] localID:fetch[1] primAccession:fetch[2]];
	    }
	  [lPolymer2 setDescription:fetch[3]];
	  [nlet2 free];
	  printf("Entry %s %s fetched. ",[lPolymer2 getSource],[lPolymer2 getLocalID]);
	  fflush(stdout);
	  if([lPolymer0 class]==[lPolymer2 class])
	    {
	      printf("Starting alignment algorithm...");
	      fflush(stdout);
	      switch(algorithmSelector)
		{
		case ALIGNALGNEEDLEWUNSCH:
		  align=[NLet needlemanWunschOf:[lPolymer0 getNLet] from:i0 to:j0 and:[lPolymer2 getNLet] from:0 to:[lPolymer2 getLength]-1 withMatrix:matrix firstGapPenalty:firstGap restGapPenalty:restGap];
		  break;
		case ALIGNALGNEIGHBOURNWUNSCH:
		  align=[NLet neighbourhoodNeedlemanWunschOf:[lPolymer0 getNLet] from:i0 to:j0 and:[lPolymer2 getNLet] from:0 to:[lPolymer2 getLength]-1 withMatrix:matrix firstGapPenalty:firstGap restGapPenalty:restGap];
		  break;
		default:
		  [ErrorHandler error:ERRMETHODNOTPRES];
		  for(i=0;i<NUMLETTER;i++)
		    free(matrixValues[i]);
		  free(matrixValues);
		  free(fetch[0]);
		  free(fetch[1]);
		  free(fetch[2]);
		  free(fetch[3]);
		  free(fetch[4]);
		  free(fetch);
		  free(sqlCommand);
		  return RETURNERROR;
		}
	      curQuality=[align getQuality];
	      fprintf(logFile,"%-10.10s",[lPolymer2 getSource]);
	      fprintf(logFile,"  ");
	      fprintf(logFile,"%-10.10s",[lPolymer2 getLocalID]);
	      fprintf(logFile,"  ");
	      fprintf(logFile,"%-42.42s",[lPolymer2 getDescription]);
	      fprintf(logFile,"  ");
	      fprintf(logFile,"%-8ld\n",curQuality);
	      fflush(logFile);
	      curLength=[curAlignmentList getLength];
	      if(curLength<alignListLength || curQuality>[[curAlignmentList getObjectAt:curLength-1] getQuality])
		 {
		   i=0;
		   j=curLength;
		   insertPos=j;
		   while(i<j)
		     {
		       threshold=i+(j-i)/2;
		       if([[curAlignmentList getObjectAt:threshold] getQuality]>curQuality)
			 i=threshold+1;
		       else
			 j=threshold;
		       insertPos=j;
		     }
		   [curSearchList insertAt:insertPos object:lPolymer2];
		   [curAlignmentList insertAt:insertPos object:align];
		   if(curLength>=alignListLength)
		     {
		       delObject=[curSearchList getObjectAt:curLength];
		       [curSearchList deleteAt:curLength];
		       [delObject free];
		       delObject=[curAlignmentList getObjectAt:curLength];
		       [curAlignmentList deleteAndDestroyAt:curLength];
		       [delObject free];
		     }
		 }
	      else
		{
		  [align free];
		  [lPolymer2 free];
		}
	      printf("finished.\n");
	    }
	  else
	    {
	      printf("No alignment algorithm performed.\n");
	      [lPolymer2 free];
	      [nlet2 free];
	    }
	}
      db_close_cursor(DEFDBCONNECTION,5);
      fprintf(logFile,"\n\n");
      fflush(logFile);
      if([curAlignmentList isEmpty])
	[ErrorHandler error:ERRDBNOFETCH];
      else
	{
	  optOutput=readCharWCommandDefault(stdin,"Alignment completed, output to file now?",'y');
	  if(toupper(optOutput)=='Y')
	    [self outputAlignments];
	}
    }
  else
    [ErrorHandler error:ERRDBCONNECTNOTPRES];
  [matrix free];
  for(i=0;i<NUMLETTER;i++)
    free(matrixValues[i]);
  free(matrixValues);
  free(fetch[0]);
  free(fetch[1]);
  free(fetch[2]);
  free(fetch[3]);
  free(fetch[4]);
  free(fetch);
  free(sqlCommand);
  return RETURNEXIT;
}

- (int)alignQueryNLetWBulkFromFile:(LinearPolymer*)lPolymer0 from:(long)i0 to:(long)j0
{
  char optFileFormat;
  int curLPType, curFileFormat;
  char *fileName;
  FILE *file;
  int algorithmSelector, matrixSelector;
  LinearPolymer *lPolymer2;
  Alignment *align;
  int **matrixValues;
  MatrixInteger *matrix;
  int firstGap, restGap;
  long alignListLength;
  int curQuality;
  long curLength, insertPos;
  long threshold;
  id delObject;
  char optOutput;
  long i, j;
  fileName=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  matrixValues=(int**)malloc(NUMLETTER*sizeof(int*));
  for(i=0;i<NUMLETTER;i++)
    matrixValues[i]=(int*)malloc(NUMLETTER*sizeof(int));
  if(!fileName || !matrixValues[NUMLETTER-1])
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(fileName);
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  if(!lPolymer0)
    {
      [ErrorHandler error:ERROBJNOTPRES];
      free(fileName);
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  readStringWCommand(fileName,stdin,"Please enter the library file name");
  file=fopen(fileName,"r");
  if(!file)
    {
      [ErrorHandler error:ERRFILEOPEN];
      free(fileName);
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  if([lPolymer0 isKindOf:[DNA class]])
    {
      curLPType=LPDNATYPE;
      while(optFileFormat!='P' && optFileFormat!='E')
	{
	  optFileFormat=readCharWCommandDefault(stdin,"Input file format: (P)lain or (E)MBL?",'P');
	  optFileFormat=toupper(optFileFormat);
	}
      if(optFileFormat=='P')
	curFileFormat=FILEPLAINDNA;
      else if(optFileFormat=='E')
	curFileFormat=FILEEMBL;
    }
  else if([lPolymer0 isKindOf:[Protein class]])
    {
      curLPType=LPPROTEINTYPE;
      while(optFileFormat!='P' && optFileFormat!='S')
	{
	  optFileFormat=readCharWCommandDefault(stdin,"Input file format: (P)lain or (S)WISS-PROT?",'P');
	  optFileFormat=toupper(optFileFormat);
	}
      if(optFileFormat=='P')
	curFileFormat=FILEPLAINPROTEIN;
      else if(optFileFormat=='S')
	curFileFormat=FILESWISSPROT;
    }
  algorithmSelector=[self chooseAlignmentAlgorithm];
  matrixSelector=[self chooseAlignmentMatrix:curLPType];
  switch(matrixSelector)
    {
    case ALMATRIXIDENTITY:
      for(i=0;i<NUMLETTER;i++)
	for(j=0;j<NUMLETTER;j++)
	  matrixValues[i][j]=identityMatrix[i][j];
      break;
    case ALMATRIXDNASTANDARD:
      for(i=0;i<NUMLETTER;i++)
	for(j=0;j<NUMLETTER;j++)
	  matrixValues[i][j]=standardDNAMatrix[i][j];
      break;
    case ALMATRIXPAM250:
      for(i=0;i<NUMLETTER;i++)
	for(j=0;j<NUMLETTER;j++)
	  matrixValues[i][j]=pam250Matrix[i][j];
      break;
    default:
      [ErrorHandler error:ERROBJNOTPRES];
      for(i=0;i<NUMLETTER;i++)
	free(matrixValues[i]);
      free(matrixValues);
      return RETURNERROR;
    }
  matrix=[MatrixInteger new:matrixValues rows:NUMLETTER columns:NUMLETTER];
  firstGap=readIntWCommand(stdin,"Please enter the penalty for opening a gap");
  restGap=readIntWCommand(stdin,"Please enter the penalty for continuing a gap");
  alignListLength=readLongWCommandDefault(stdin,"Please enter the number of alignments kept to be output",5000);
  fprintf(logFile,"\n");
  fprintf(logFile,"COMPUTED ALIGNMENTS");
  fprintf(logFile,"\n\n");
  fprintf(logFile,"Query linear polymer: %s %s\n",[lPolymer0 getSource],[lPolymer0 getLocalID]);
  fprintf(logFile,"Description: %.65s\n",[lPolymer0 getDescription]);
  fprintf(logFile,"Length: %ld\n",[lPolymer0 getLength]);
  fflush(logFile);
  fprintf(logFile,"\n\n");
  fprintf(logFile,"Alingnment summary:\n");
  fprintf(logFile,"\n");
  fflush(logFile);
  curQueryLPolymer=[linPolymerList getFirstPosOf:lPolymer0];
  [curSearchList empty];
  [curAlignmentList emptyAndDestroy];
  while((lPolymer2=[self fileFetch:file type:curLPType format:curFileFormat])!=nil)
    {
      printf("Entry %s %s fetched. ",[lPolymer2 getSource],[lPolymer2 getLocalID]);
      fflush(stdout);
      if([lPolymer0 class]==[lPolymer2 class])
	{
	  printf("Starting alignment algorithm...");
	  fflush(stdout);
	  switch(algorithmSelector)
	    {
	    case ALIGNALGNEEDLEWUNSCH:
	      align=[NLet needlemanWunschOf:[lPolymer0 getNLet] from:i0 to:j0 and:[lPolymer2 getNLet] from:0 to:[lPolymer2 getLength]-1 withMatrix:matrix firstGapPenalty:firstGap restGapPenalty:restGap];
	      break;
	    case ALIGNALGNEIGHBOURNWUNSCH:
	      align=[NLet neighbourhoodNeedlemanWunschOf:[lPolymer0 getNLet] from:i0 to:j0 and:[lPolymer2 getNLet] from:0 to:[lPolymer2 getLength]-1 withMatrix:matrix firstGapPenalty:firstGap restGapPenalty:restGap];
	      break;
	    default:
	      [ErrorHandler error:ERRMETHODNOTPRES];
	      for(i=0;i<NUMLETTER;i++)
		free(matrixValues[i]);
	      free(matrixValues);
	      return RETURNERROR;
	    }
	  curQuality=[align getQuality];
	  fprintf(logFile,"%-10.10s",[lPolymer2 getSource]);
	  fprintf(logFile,"  ");
	  fprintf(logFile,"%-10.10s",[lPolymer2 getLocalID]);
	  fprintf(logFile,"  ");
	  fprintf(logFile,"%-42.42s",[lPolymer2 getDescription]);
	  fprintf(logFile,"  ");
	  fprintf(logFile,"%-8ld\n",curQuality);
	  fflush(logFile);
	  curLength=[curAlignmentList getLength];
	  if(curLength<alignListLength || curQuality>[[curAlignmentList getObjectAt:curLength-1] getQuality])
	    {
	      i=0;
	      j=curLength;
	      insertPos=j;
	      while(i<j)
		{
		  threshold=i+(j-i)/2;
		  if([[curAlignmentList getObjectAt:threshold] getQuality]>curQuality)
		    i=threshold+1;
		  else
		    j=threshold;
		  insertPos=j;
		}
	      [curSearchList insertAt:insertPos object:lPolymer2];
	      [curAlignmentList insertAt:insertPos object:align];
	      if(curLength>=alignListLength)
		{
		  delObject=[curSearchList getObjectAt:curLength];
		  [curSearchList deleteAt:curLength];
		  [delObject free];
		  delObject=[curAlignmentList getObjectAt:curLength];
		  [curAlignmentList deleteAndDestroyAt:curLength];
		  [delObject free];
		}
	    }
	  else
	    {
	      [align free];
	      [lPolymer2 free];
	    }
	  printf("finished.\n");
	}
      else
	{
	  printf("No alignment algorithm performed.\n");
	  [lPolymer2 free];
	}
    }
  fprintf(logFile,"\n\n");
  fflush(logFile);
  if(![curAlignmentList isEmpty])
    {
      optOutput=readCharWCommandDefault(stdin,"Alignment completed, output to file now?",'y');
      if(toupper(optOutput)=='Y')
	[self outputAlignments];
    }
  [matrix free];
  free(fileName);
  for(i=0;i<NUMLETTER;i++)
    free(matrixValues[i]);
  free(matrixValues);
  return RETURNEXIT;
}

- (LinearPolymer*)fileFetch:(FILE*)file0 type:(int)type0 format:(int)format0
{
  LinearPolymer *newLPolymer;
  newLPolymer=nil;
  if(type0==LPDNATYPE)
    switch(format0)
      {
      case FILEPLAINDNA:
	newLPolymer=[self inputPlainDNAFile:file0];
	break;
      case FILEEMBL:
	newLPolymer=[self inputEMBLFile:file0];
	break;
      default:
	[ErrorHandler error:ERRFILEFORMATSUPP];
	return nil;
      }
  else if(type0==LPPROTEINTYPE)
    switch(format0)
      {
      case FILEPLAINPROTEIN:
	newLPolymer=[self inputPlainProteinFile:file0];
	break;
      case FILESWISSPROT:
	newLPolymer=[self inputSWISSPROTFile:file0];
	break;
      default:
	[ErrorHandler error:ERRFILEFORMATSUPP];
	return nil;
      }
  return newLPolymer;
}

- (int)outputAlignments
{
  char *fileName;
  FILE *file;
  long numOutputAlign, count;
  ArrayDoubleWStatistics *statistics;
  LinkedListElement *curAlignment, *curSearchPartner;
  int curQuality, maxColQuality;
  long curLength;
  long i;
  int j, l;
  fileName=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!fileName)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  if([curAlignmentList getLength])
    {
      readStringWCommand(fileName,stdin,"Please enter the file name");
      file=fopen(fileName,"w");
      if(!file)
	{
	  [ErrorHandler error:ERRFILEOPEN];
	  return RETURNERROR;
	}
      numOutputAlign=readLongWCommandDefault(stdin,"Number of alignments to be displayed",([curAlignmentList getLength]+9)/10);
      fprintf(file,"\n");
      fprintf(file,"RESULT FILE FOR PERFORMED ALIGNMENT ALGORITHM\n");
      fprintf(file,"\n\n");
      fprintf(file,"Query linear polymer: %s %s\n",[[linPolymerList getObjectAt:curQueryLPolymer] getSource],[[linPolymerList getObjectAt:curQueryLPolymer] getLocalID]);
      fprintf(file,"Description: %.65s\n",[[linPolymerList getObjectAt:curQueryLPolymer] getDescription]);
      fprintf(file,"Length: %ld\n",[[linPolymerList getObjectAt:curQueryLPolymer] getLength]);
      statistics=[ArrayDoubleWStatistics new];
      curAlignment=[curAlignmentList getHead];
      while(curAlignment)
	{
	  [statistics append:[[curAlignment getObject] getQuality]];
	  curAlignment=[curAlignment getNext];
	}
      fprintf(file,"Number of alignments: %d\n",[statistics getSize]);
      fprintf(file,"Average alignment quality: %.2f\n",[statistics getMean]);
      fprintf(file,"Standard deviation: %.2f\n",[statistics getStdDeviation]);
      fprintf(file,"\n\n");
      fprintf(file,"Alingnment summary:\n");
      fprintf(file,"\n");
      count=0;
      curSearchPartner=[curSearchList getHead];
      curAlignment=[curAlignmentList getHead];
      while(curSearchPartner && count<numOutputAlign)
	{
	  fprintf(file,"%-10.10s",[[curSearchPartner getObject] getSource]);
	  fprintf(file,"  ");
	  fprintf(file,"%-10.10s",[[curSearchPartner getObject] getLocalID]);
	  fprintf(file,"  ");
	  fprintf(file,"%-42.42s",[[curSearchPartner getObject] getDescription]);
	  fprintf(file,"  ");
	  curQuality=[[curAlignment getObject] getQuality];
	  fprintf(file,"%-8ld\n",curQuality);
	  curSearchPartner=[curSearchPartner getNext];
	  curAlignment=[curAlignment getNext];
	  count++;
	}
      fprintf(file,"\n\n");
      fprintf(file,"Alignment details:\n");
      fprintf(file,"\n");
      count=0;
      curSearchPartner=[curSearchList getHead];
      curAlignment=[curAlignmentList getHead];
      while(curSearchPartner && count<numOutputAlign)
	{
	  maxColQuality=0;
	  for(i=0;i<[[curAlignment getObject] getLength];i++)
	    if([[curAlignment getObject] getQualityRowAtColumn:i]>maxColQuality)
	      maxColQuality=[[curAlignment getObject] getQualityRowAtColumn:i];
	  fprintf(file,"Partner: %s %s\n",[[curSearchPartner getObject] getSource],[[curSearchPartner getObject] getLocalID]);
	  fprintf(file,"Description: %.65s\n",[[curSearchPartner getObject] getDescription]);
	  fprintf(file,"Length: %ld\n",[[curSearchPartner getObject] getLength]);
	  curQuality=[[curAlignment getObject] getQuality];
	  fprintf(file,"Quality of alignment: %ld\n",curQuality);
	  fprintf(file,"\n");
	  i=0;
	  curLength=[[curAlignment getObject] getLength];
	  while(60*i<curLength)
	    {
	      l=curLength-60*i;
	      if(l>60)
		l=60;
	      fprintf(file,"%-10.10s",[[linPolymerList getObjectAt:curQueryLPolymer] getLocalID]);
	      fprintf(file,"  ");
	      fprintf(file,"%s\n",[[curAlignment getObject] getRow:0 from:60*i to:60*i+l-1]);
	      fprintf(file,"            ");
	      for(j=0;j<l;j++)
		if([[curAlignment getObject] getQualityRowAtColumn:60*i+j]==maxColQuality)
		  fprintf(file,"*");
		else if([[curAlignment getObject] getQualityRowAtColumn:60*i+j]>0)
		  fprintf(file,"|");
		else
		  fprintf(file," ");
	      fprintf(file,"\n");
	      fprintf(file,"%-10.10s",[[curSearchPartner getObject] getLocalID]);
	      fprintf(file,"  ");
		  fprintf(file,"%s\n",[[curAlignment getObject] getRow:1 from:60*i to:60*i+l-1]);
	      fprintf(file,"  ");
	      fprintf(file,"\n");
	      i++;
	    }
	  curSearchPartner=[curSearchPartner getNext];
	  curAlignment=[curAlignment getNext];
	  count++;
	}
      [statistics free];
      fclose(file);
    }
  else
    [ErrorHandler error:ERROBJNOTPRES];
  free(fileName);
  return RETURNEXIT;
}

- (int)outputLinearPolymer
{
  int num;
  id curLPolymer;
  char optDestination;
  num=[self chooseLinearPolymer:LPNOTYPE];
  if(num==RETURNERROR)
    return RETURNERROR;
  curLPolymer=[linPolymerList getObjectAt:num];
  if(useDB)
    while(optDestination!='D' && optDestination!='F')
      {
	optDestination=readCharWCommandDefault(stdin,"Output a linear polymer to (D)atabase or (F)ile?",'D');
	optDestination=toupper(optDestination);
      }
  else
    optDestination='F';
  switch(optDestination)
    {
    case 'D':
      break;
    case 'F':
      return [self outputLinearPolymerToFile:curLPolymer];
      break;
    default:
      break;
    }
  return RETURNEXIT;
}

- (int)outputLinearPolymerToFile:(LinearPolymer*)lPolymer0
{
  char *fileName;
  FILE *file;
  char optFileFormat;
  int returnCode;
  fileName=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!fileName)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return RETURNERROR;
    }
  readStringWCommand(fileName,stdin,"Please enter the file name");
  file=fopen(fileName,"w");
  if(!file)
    {
      [ErrorHandler error:ERRFILEOPEN];
      return RETURNERROR;
    }
  if([lPolymer0 isKindOf:[DNA class]])
    {
      while(optFileFormat!='P' && optFileFormat!='E')
	{
	  optFileFormat=readCharWCommandDefault(stdin,"Output file format: (P)lain or (E)MBL?",'P');
	  optFileFormat=toupper(optFileFormat);
	}
      if(optFileFormat=='P')
	returnCode=[self output:(DNA*)lPolymer0 toPlainDNAFile:file];
      else if(optFileFormat=='E')
	returnCode=[self output:(DNA*)lPolymer0 toEMBLFile:file];
    }
  else if([lPolymer0 isKindOf:[Protein class]])
    {
      while(optFileFormat!='P' && optFileFormat!='S')
	{
	  optFileFormat=readCharWCommandDefault(stdin,"Output file format: (P)lain or (S)WISS-PROT?",'P');
	  optFileFormat=toupper(optFileFormat);
	}
      if(optFileFormat=='P')
	returnCode=[self output:(Protein*)lPolymer0 toPlainProteinFile:file];
      else if(optFileFormat=='S')
	returnCode=[self output:(Protein*)lPolymer0 toSWISSPROTFile:file];
      else
	{
	  free(fileName);
	  fclose(file);
	  return RETURNERROR;
	}
    }
  free(fileName);
  fclose(file);
  return returnCode;
}

- (int)output:(DNA*)dna0 toPlainDNAFile:(FILE*)file0
{
  long length, i;
  length=[dna0 getLength];
  i=0;
  while(i<length)
    {
      fprintf(file0,"%c",[dna0 getSequenceAt:i++]);
      if(!(i%TXTMODECHARPERLINE))
	fprintf(file0,"\n");
    }
  fprintf(file0,"\n");
  return RETURNEXIT;
}

- (int)output:(DNA*)dna0 toEMBLFile:(FILE*)file0
{
  long length, others, i;
  int lineFill, j;
  fprintf(file0,"ID   ");
  fprintf(file0,"%s\n",[dna0 getLocalID]);
  fprintf(file0,"AC   ");
  fprintf(file0,"%s;\n",[dna0 getPrimAccession]);
  fprintf(file0,"DE   ");
  fprintf(file0,"%-72.72s\n",[dna0 getDescription]);
  length=[dna0 getLength];
  others=0;
  for(i=0;i<SIZEASCII;i++)
    if(islower(i) && ![DNANLet isDNANucleotide:i])
      others+=[dna0 getCompositionFor:i];
  fprintf(file0,"SQ   ");
  fprintf(file0,"Sequence  %ld BP;",length);
  fprintf(file0,"  %ld A;  %ld C;  %ld G;  %ld T;",[dna0 getCompositionFor:'a'],[dna0 getCompositionFor:'c'],[dna0 getCompositionFor:'g'],[dna0 getCompositionFor:'t']);
  fprintf(file0,"  %ld other;\n",others);
  i=0;
  while(i<length)
    switch(i%60)
      {
      case 0:
	fprintf(file0,"     %c",[dna0 getSequenceAt:i++]);
	break;
      case 59:
	fprintf(file0,"%c  %8ld\n",[dna0 getSequenceAt:i],i+1);
	i++;
	break;
      default:
	if(!(i%10))
	  fprintf(file0," %c",[dna0 getSequenceAt:i++]);
	else
	  fprintf(file0,"%c",[dna0 getSequenceAt:i++]);
	break;
      }
  lineFill=60-i%60;
  if(lineFill!=60)
    {
      for(j=0;j<lineFill;j++)
	fprintf(file0," ");
      for(j=0;j<lineFill/10;j++)
	fprintf(file0," ");
      fprintf(file0,"  %8ld\n",i);
    }
  fprintf(file0,"//\n");
  return RETURNEXIT;
}

- (int)output:(Protein*)protein0 toPlainProteinFile:(FILE*)file0
{
  long length, i;
  length=[protein0 getLength];
  i=0;
  while(i<length)
    {
      fprintf(file0,"%c",[protein0 getSequenceAt:i++]);
      if(!(i%TXTMODECHARPERLINE))
	fprintf(file0,"\n");
    }
  fprintf(file0,"\n");
  return RETURNEXIT;
}

- (int)output:(Protein*)protein0 toSWISSPROTFile:(FILE*)file0
{
  long length, others, i;
  int lineFill, j;
  fprintf(file0,"ID   ");
  fprintf(file0,"%s\n",[protein0 getLocalID]);
  fprintf(file0,"AC   ");
  fprintf(file0,"%s;\n",[protein0 getPrimAccession]);
  fprintf(file0,"DE   ");
  fprintf(file0,"%-72.72s\n",[protein0 getDescription]);
  length=[protein0 getLength];
  fprintf(file0,"SQ   ");
  fprintf(file0,"Sequence  %ld AA;\n",length);
  i=0;
  while(i<length)
    switch(i%60)
      {
      case 0:
	fprintf(file0,"     %c",[protein0 getSequenceAt:i++]);
	break;
      case 59:
	fprintf(file0,"%c  %8ld\n",[protein0 getSequenceAt:i],i+1);
	i++;
	break;
      default:
	if(!(i%10))
	  fprintf(file0," %c",[protein0 getSequenceAt:i++]);
	else
	  fprintf(file0,"%c",[protein0 getSequenceAt:i++]);
	break;
      }
  lineFill=60-i%60;
  if(lineFill!=60)
    {
      for(j=0;j<lineFill;j++)
	fprintf(file0," ");
      for(j=0;j<lineFill/10;j++)
	fprintf(file0," ");
      fprintf(file0,"  %8ld\n",i);
    }
  fprintf(file0,"//\n");
  return RETURNEXIT;
}

- (int)closeLinearPolymer
{
  int num;
  num=0;
  num=[self chooseLPolymerFromList];
  [linPolymerList deleteAndDestroyAt:num];
  return RETURNEXIT;
}

- (int)dbConnect
{
  char *user, *passwd;
  user=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  passwd=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  if(!user || !passwd)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      free(user);
      free(passwd);
      return RETURNERROR;
    }
  if(!useDB)
    {
      readLoginData(user,passwd);
      if(db_connect(DEFDBCONNECTION,user,passwd,dbServer)==RETURNEXIT)
	{
	  useDB=YES;
	  free(user);
	  free(passwd);
	  return RETURNEXIT;
	}
    }
  else
    [ErrorHandler error:ERRDBCONNECTEXIST];
  free(user);
  free(passwd);
  return RETURNERROR;
}

- (int)dbDisconnect
{
  if(useDB)
    if(db_disconnect(DEFDBCONNECTION)==RETURNEXIT)
      {
	useDB=NO;
	return RETURNEXIT;
      }
    else
      return RETURNERROR;
  else
    {
      [ErrorHandler error:ERRDBCONNECTNOTPRES];
      return RETURNERROR;
    }      
}

-(int)openUNIXShellCommand
{
  char *shell;
  shell=(char*)malloc(MINSTRINGLEN*sizeof(char));
  readStringWCommandDefault(shell,stdin,"Please enter the shell name / command","bash");
  system(shell);
  free(shell);
  return RETURNEXIT;
}

- free
{
  free(dbServer);
  [linPolymerList free];
  [curSearchList free];
  [curAlignmentList free];
  return [super free];
}

@end
