/* Name: textif.m                                                            **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse TextInterface, die eine Benutzer-     **
**           schnittstelle im Textmodus bereitstellt                         **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 19.10.1995                                                      **
** Zuletzt bearbeitet: 17.07.1998                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "textif.h"

#include "error.h"
#include "constant.h"

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/* Klasse: TextInterface                                                     */

@implementation TextInterface

+ new
{
  return [[self alloc] init];
}

+ alloc
{
  TextInterface *newObj;
  newObj=[super alloc];
  newObj->usrInput=(char*)malloc(DEFSTRINGLEN*sizeof(char));
  newObj->action=(char*)malloc(MINSTRINGLEN*sizeof(char));
  if(!newObj || !newObj->usrInput || !newObj->action)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

- init
{
  self=[super init];
  return self;
}

- (char*)getAction
{
  char* returnAction;
  returnAction=(char*)malloc(strlen(action)*sizeof(char));
  if(!returnAction)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return DEFPOINTER;
    }
  strcpy(returnAction,action);
  return returnAction;
}

- (int)menuSelector:(int)menu0
{
  switch(menu0)
    {
    case MENUMAIN:
      return [self mainMenu];
      break;
    case MENU2:
      return [self menu2];
      break;
    case MENU3:
      return [self menu3];
      break;
    case MENU6:
      return [self menu6];
      break;
    default:
      return [self mainMenu];
      break;
    }
  return [self mainMenu];
}

- (int)mainMenu
{
  /* Menuetitel */
  outputMenuTitle("Main Menu");
  /* Menueinformation */
  outputMenuDescription("This is the main menu. You can choose to open, edit, output or close sequences or to perform a database search.");
  /* Menuepunkte */
  outputMenuItem(1,'I',SHORTINPUTLP,"Input a linear polymer");
  outputMenuItem(2,'E',SHORTNOACTION,"Edit and analyse a linear polymer");
  outputMenuItem(3,'S',SHORTNOACTION,"Perform an alignment algorithm");
  outputMenuItem(4,'O',SHORTOUTPUTLP,"Output a linear polymer");
  outputMenuItem(5,'C',SHORTCLOSELP,"Close a linear polymer");
  outputMenuItem(6,'D',SHORTNOACTION,"Communicate to the database");
  outputMenuItem(7,'U',SHORTUNIXSH,"Open a (UNIX-)shell / Execute a command");
  outputMenuItem(8,'L',SHORTLISTLP,"List all open linear polymers");
  outputMenuItem(0,'X',SHORTEXIT,"Exit");
  /* Einlesen der Auswahl */
  readMenuChoice(usrInput);
  /* Ausloesen der Aktionen */
  /* 1. Fall: direkt ueber Programmkuerzel */
  if(strlen(usrInput)>=DEFSHORTHANDLEN)
    {
      strncpy(action,usrInput,DEFSHORTHANDLEN);
      return MENUMAIN;
    }
  /* 2. Fall: ueber Menuepunkte */
  else
    switch(toupper(usrInput[0]))
      {
      case '1' : case 'I' :
	strncpy(action,SHORTINPUTLP,DEFSHORTHANDLEN);
	return MENUMAIN;
      case '2' : case 'E' :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENU2;
      case '3' : case 'S' :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENU3;
      case '4' : case 'O' :
	strncpy(action,SHORTOUTPUTLP,DEFSHORTHANDLEN);
	return MENUMAIN;
      case '5' : case 'C' :
	strncpy(action,SHORTCLOSELP,DEFSHORTHANDLEN);
	return MENUMAIN;
      case '6' : case 'D' :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENU6;
      case '7' : case 'U' :
	strncpy(action,SHORTUNIXSH,DEFSHORTHANDLEN);
	return MENUMAIN;
      case '8' : case 'L' :
	strncpy(action,SHORTLISTLP,DEFSHORTHANDLEN);
	return MENUMAIN;
      case '0' : case 'X' :
	strncpy(action,SHORTEXIT,DEFSHORTHANDLEN);
	return MENUMAIN;
      default :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENUMAIN;
      }
  return RETURNERROR;
}

- (int)menu2
{
  outputMenuTitle("Edit and analyse a linear polymer");
  outputMenuDescription("With this menu you can edit some points concerning linear polymers, perform translations and retranslations and get additional information about the linear polymers.");
  outputMenuItem(1,'S',SHORTEDITSOURCE,"Edit the source of a linear polymer");
  outputMenuItem(2,'I',SHORTEDITLOCALID,"Edit the local ID of a linear polymer");
  outputMenuItem(3,'D',SHORTEDITDESCRIPTION,"Edit the description of a linear polymer");
  outputMenuItem(4,'G',SHORTGENERATERANDOM,"Generate a random linear polymer");
  outputMenuItem(5,'C',SHORTLISTCOMPOSITION,"List the composition of a linear polymer");
  outputMenuItem(6,'A',SHORTANTISENSE,"DNA: Generate the antisense strand");
  outputMenuItem(7,'T',SHORTTRANSLATE,"DNA: Translate into protein");
  outputMenuItem(8,'V',SHORTDNAREVERSE,"DNA: Reverse a sequence");
  outputMenuItem(9,'P',SHORTTRIPLETREVERSE,"DNA: Reverse a sequence by triplets");
  outputMenuItem(10,'N',SHORTMININFORMATION,"DNA: Compute the minimum information");
  outputMenuItem(11,'E',SHORTDENSITY,"DNA: Get the information density");
  outputMenuItem(12,'R',SHORTRETRANSLATE,"Protein: Retranslate into DNA");
  outputMenuItem(13,'L',SHORTLISTLP,"List all open linear polymers");
  outputMenuItem(0,'M',SHORTNOACTION,"Back to the main menu");
  readMenuChoice(usrInput);
  if(strlen(usrInput)>=DEFSHORTHANDLEN)
    {
      strncpy(action,usrInput,DEFSHORTHANDLEN);
      return MENU2;
    }
  else
    switch(toupper(usrInput[0]))
      {
      case '1' : 
	if(!isdigit(usrInput[1]))
	  {
	    strncpy(action,SHORTEDITSOURCE,DEFSHORTHANDLEN);
	    return MENU2;
	  }
	else
	  switch(usrInput[1])
	    {
	    case '0' :
	      strncpy(action,SHORTMININFORMATION,DEFSHORTHANDLEN);
	      return MENU2;
	    case '1' :
	      strncpy(action,SHORTDENSITY,DEFSHORTHANDLEN);
	      return MENU2;
	    case '2' :
	      strncpy(action,SHORTRETRANSLATE,DEFSHORTHANDLEN);
	      return MENU2;
	    case '3' :
	      strncpy(action,SHORTLISTLP,DEFSHORTHANDLEN);
	      return MENU2;
	    default :
	      strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	      return MENU2;
	    }
      case 'S' :
	strncpy(action,SHORTEDITSOURCE,DEFSHORTHANDLEN);
	return MENU2;
      case '2' : case 'I' :
	strncpy(action,SHORTEDITLOCALID,DEFSHORTHANDLEN);
	return MENU2;
      case '3' : case 'D' :
	strncpy(action,SHORTEDITDESCRIPTION,DEFSHORTHANDLEN);
	return MENU2;
      case '4' : case 'G' :
	strncpy(action,SHORTGENERATERANDOM,DEFSHORTHANDLEN);
	return MENU2;
      case '5' : case 'C' :
	strncpy(action,SHORTLISTCOMPOSITION,DEFSHORTHANDLEN);
	return MENU2;
      case '6' : case 'A' :
	strncpy(action,SHORTANTISENSE,DEFSHORTHANDLEN);
	return MENU2;
      case '7' : case 'T' :
	strncpy(action,SHORTTRANSLATE,DEFSHORTHANDLEN);
	return MENU2;
      case '8' : case 'V' :
	strncpy(action,SHORTDNAREVERSE,DEFSHORTHANDLEN);
	return MENU2;
      case '9' : case 'P' :
	strncpy(action,SHORTTRIPLETREVERSE,DEFSHORTHANDLEN);
	return MENU2;
      case '0' : case 'M' :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENUMAIN;
      case 'N' :
	strncpy(action,SHORTMININFORMATION,DEFSHORTHANDLEN);
	return MENU2;
      case 'E' :
	strncpy(action,SHORTDENSITY,DEFSHORTHANDLEN);
	return MENU2;
      case 'R' :
	strncpy(action,SHORTRETRANSLATE,DEFSHORTHANDLEN);
	return MENU2;
      case 'L' :
	strncpy(action,SHORTLISTLP,DEFSHORTHANDLEN);
	return MENU2;
      default :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENU2;
      }
  return RETURNERROR;
}

- (int)menu3
{
  outputMenuTitle("Perform an alignment algorithm");
  outputMenuDescription("With this menu you can perform various alignment algorithms.");
  outputMenuItem(1,'T',SHORTALIGNTWONLETS,"Align two n-lets");
  outputMenuItem(2,'A',SHORTALIGNQUERYWBULK,"Align query n-let with a bulk of n-lets");
  outputMenuItem(3,'O',SHORTOUTPUTALIGN,"Output current alignment list to a file");
  outputMenuItem(4,'L',SHORTLISTLP,"List all open linear polymers");
  outputMenuItem(0,'M',SHORTNOACTION,"Back to the main menu");
  readMenuChoice(usrInput);
  if(strlen(usrInput)>=DEFSHORTHANDLEN)
    {
      strncpy(action,usrInput,DEFSHORTHANDLEN);
      return MENU3;
    }
  else
    switch(toupper(usrInput[0]))
      {
      case '1' : case 'T' :
	strncpy(action,SHORTALIGNTWONLETS,DEFSHORTHANDLEN);
	return MENU3;
      case '2' : case 'A' :
	strncpy(action,SHORTALIGNQUERYWBULK,DEFSHORTHANDLEN);
	return MENU3;
      case '3' : case 'O' :
	strncpy(action,SHORTOUTPUTALIGN,DEFSHORTHANDLEN);
	return MENU3;
      case '4' : case 'L' :
	strncpy(action,SHORTLISTLP,DEFSHORTHANDLEN);
	return MENU3;
      case '0' : case 'M' :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENUMAIN;
      default :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENU3;
      }
  return RETURNERROR;
}

- (int)menu6
{
  outputMenuTitle("Communicate to the database");
  outputMenuDescription("With this menu you can open and close a connection to the database.");
  /* outputMenuItem(1,'C',SHORTDBCONNECT,"Connect to the database");
  outputMenuItem(2,'D',SHORTDBDISCONNECT,"Disconnect from the database");
  outputMenuItem(3,'L',SHORTLISTLP,"List all open linear polymers"); */
  outputMenuItem(0,'M',SHORTNOACTION,"Back to the main menu");
  readMenuChoice(usrInput);
  if(strlen(usrInput)>=DEFSHORTHANDLEN)
    {
      strncpy(action,usrInput,DEFSHORTHANDLEN);
      return MENU6;
    }
  else
    switch(toupper(usrInput[0]))
      {
	/* case '1' : case 'C' :
	strncpy(action,SHORTDBCONNECT,DEFSHORTHANDLEN);
	return MENU6;
      case '2' : case 'D' :
	strncpy(action,SHORTDBDISCONNECT,DEFSHORTHANDLEN);
	return MENU6;
      case '3' : case 'L' :
	strncpy(action,SHORTLISTLP,DEFSHORTHANDLEN);
	return MENU6; */
      case '0' : case 'M' :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENUMAIN;
      default :
	strncpy(action,SHORTNOACTION,DEFSHORTHANDLEN);
	return MENU6;
      }
  return RETURNERROR;
}

- free
{
  free(usrInput);
  free(action);
  return [super free];
}

@end
